
from random import choice,randint
import os

def number_maker():
        pidh = ['911','912','933','936','937','938','939','902','990']
        number = f"0{choice(pidh)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}"
        return number
    
def get_random_email():
    path = os.path.abspath(os.path.join(os.path.dirname(__file__)))+'/emails.txt'
    with open(path , 'r') as f:
        res = f.read()
    return choice(res.split('\n'))
 



class Gate:
    def __init__(self,gate_key,test_mode,**kargs) -> None:
        self.gate_key = gate_key
        self.test_mode = test_mode
        callback_path = 'callback'
        base_callback_url:str = kargs.get('base_callback_url')
        self.callback = base_callback_url.rstrip('/')+f"/{callback_path}/"+"{}"

        for k,v in kargs.items():
            self.__dict__[k]=v
    
    def callback_maker(self,order_id):
        return self.callback.format(str(order_id))
         
    def pay_link(self,**kargs):
        '''must be overriding'''
        pass
    def verify(self,**kargs):
        '''must be overriding'''
        pass


import requests,json
from .utils import number_maker,get_random_email,Gate


class gateway(Gate):
    def __init__(self ,**kargs) -> None:
        super().__init__(**kargs)
        if self.test_mode:
            self.gate_key='sandbox'
 
    def pay_link(self,**args):
        amount,callback,order_id = args.get('amount') , args.get('callback'), args.get('order_id')
        if not callback:
            callback = self.callback_maker(order_id)

        mobile,email = args.get('mobile',str(number_maker())) , args.get('email',str(get_random_email()))
        data = {
            'pin' : self.gate_key ,
            'amount' : amount,
            'callback' : callback,
            'mobile':mobile,
            'email':email
            }
        response = requests.post('https://panel.aqayepardakht.ir/api/v2/create', data = data,timeout=20)
        json_data = json.loads(response.text)
        if response.status_code == 200 and json_data['status'] == 'success':
            path = 'startpay'
            if self.test_mode:path='startpay/sandbox'
            return f'https://panel.aqayepardakht.ir/{path}/'+json_data['transid']
        return None            
    
    def verify(self,**args):
        amount,transid = args.get('amount') , args.get('transid')
        data = {
            'pin' : self.gate_key,
            'amount' : amount,
            'transid' : transid
        }

        response = requests.post('https://panel.aqayepardakht.ir/api/v2/verify', data = data,timeout=20)
        json_data = json.loads(response.text)
        if response.status_code == 200 and str(json_data['code']) == '1':
            return True ,args
        return False  ,args


from telegram.utils.request import Request
from telegram.ext import Updater, Filters
from telegram.ext import MessageHandler, ConversationHandler
import json
from threading import Thread
from .util.filteres import RoleFilter
from .classes import Base, Conversation
from .util.enum import ConversationNode
from ...utils.scheduler import Scheduler
from telegram import Bot, Update
from starco.debug import Debug
from starco.db import DB


class TlgBot:
    def __init__(
        self, token: str, super_admin: int,
            db_config: dict[dict] = {},
            db_tables_need_init: dict[dict] = {},
            webhook_url: str = None, port: int = 443,
            ssl_key_path: str = None, ssl_cert_path: str = None,
            db_relative_path: str = '.', debug_relative_path: str = '.',
            debug_mode=False, host_id: str = None,
            proxy=None, scheduler_status: bool = True,
            edit_mode: bool = False,
            editors_id:list=[],
            respond_bad_order=True,
    ) -> None:
        '''
        '''
        self.token = token
        self.respond_bad_order=respond_bad_order
        self.edit_mode = edit_mode
        self.editors_id=editors_id
        
        self.super_admin = super_admin
        self.webhook_url = webhook_url
        self.port = port
        self.ssl_key_path = ssl_key_path
        self.ssl_cert_path = ssl_cert_path
        self.host_id = host_id
        self.db_relative_path = db_relative_path
        self.db_type = 'sqlite'
        self.debug_relative_path = debug_relative_path
        self.debug_mode = debug_mode
        self.debug = Debug(debug_mode=debug_mode,
                           relative_path=debug_relative_path)
        request_kwargs = None
        if proxy:
            self.bot = Bot(self.token, request=Request(proxy_url=proxy))
            request_kwargs = {'proxy_url': proxy}
        else:
            self.bot = Bot(self.token)
        
        self.updater = Updater(self.token, use_context=True,
                               request_kwargs=request_kwargs)
        self.dp = self.updater.dispatcher
        self.bot_username = self.dp.bot.get_me().username
        
        self.db_config = db_config
        self.db_tables_need_init = db_tables_need_init
        self.init_db()
        self.scheduler_status = scheduler_status
        self.scheduler = Scheduler(self)

    def init_db(self):
        self.forced_tables_init()
        self.db = DB(self.db_config, relative_path=self.db_relative_path,
                     debug_mode=self.debug_mode, debug_relative_path=self.debug_relative_path)
        self.need_init_tables()
        self.settings_subset_update()

    def forced_tables_init(self):
        users_cfg = {'id': 0, 'name': '', 'last_name': '', 'username': '', 'role': 0, 'editor': 0,
                     'status': 0, 'phone': 0, 'language': 0, 'get_alarm': 0, 'is_online': 0, 'time': 0, 'last_seen': 0}
        setting_cfg = {'key': '', 'value': '',
                       'unit': '', 'type': '', 'subset': '','hide':0,'toggle':0}
        texts_cfg = {'id': 0, 'key': '', 'value': '', 'language': 0, 'role': 0}
        media_cfg = {'id': 0, 'key': '', 'msg_id': '',
                     'channel_id': '', 'language': 0, 'role': 0}

        def check(table_name, cfg):
            if table_name not in self.db_config:
                self.db_config[table_name] = cfg
            else:
                for k, v in cfg.items():
                    if k not in self.db_config[table_name]:
                        self.db_config[table_name][k] = v
        check('users', users_cfg)
        check('setting', setting_cfg)
        check('texts', texts_cfg)
        check('media', media_cfg)

    def need_init_tables(self):
        tables_data = self.db_tables_need_init
        defulat_setting = {}
        setting = {**tables_data.get('setting', {}), **defulat_setting}
        tables_data['setting'] = setting
        if tables_data:

            for table in tables_data:
                res = self.db.do(table)
                keys = [i['key'] for i in res]
                for k, v in tables_data[table].items():
                    if k not in keys:
                        self.db.do(table, {**{'key': k}, **v})

                tables_in_db = self.db.do(table)
                keys = [i['key'] for i in tables_in_db]
                must_be_remove = list(
                    set(keys) - set(tables_data[table].keys()))
                for k in must_be_remove:
                    self.db.do(table, condition=f"key='{k}'", delete=True)

    def settings_subset_update(self):
        tables_data = self.db_tables_need_init
        setting_init = tables_data.get('setting', {})
        if setting_init:
            for key in setting_init:
                sub = setting_init[key]['subset']
                hide = setting_init[key]['hide']
                toggle = setting_init[key]['toggle']
                self.db.do('setting', {'subset': sub,'hide': hide,'toggle': toggle},
                           condition=f"key='{key}'")

    def add(self, class_item: Base):

        if class_item.type == 'Conversation':
            item: Conversation = class_item
            for node in item.nodes:
                node: ConversationNode = node
                self.dp.add_handler(ConversationHandler(
                    entry_points=node.entries,
                    states=node.states,
                    fallbacks=node.fallbacks,
                    allow_reentry=node.arges.get('allow_reentry', True),
                    **node.arges
                ))
            if self.respond_bad_order:
                self.dp.add_handler(MessageHandler(
                    Filters.all & RoleFilter(item) , item.add_method(item.not_fount)))

    def add_schedual_action(self,func,run_evry_sec:int,first_run=False):
        self.scheduler.add(func=func,run_evry_sec=run_evry_sec,first_run=first_run)

    def befor_run_action(self):
        self.init_db()
        if self.scheduler_status:
            Thread(target=self.scheduler.run).start()

    def init_webhook(self):
        self.bot.deleteWebhook()
        print(self.bot_username)
        self.befor_run_action()

        webhook_url = self.webhook_url
        if self.port not in [443, 80]:
            webhook_url += f':{self.port}'
        webhook_url += f'/webhook/{self.token}'
        print(webhook_url)
        self.bot.setWebhook(webhook_url, max_connections=100)

    def run_poll(self):
        self.bot.deleteWebhook()
        print(self.bot_username)
        self.befor_run_action()
        self.updater.start_polling()
        self.updater.idle()

    def webhook(self, json_data):
        try:
            data = json.loads(json_data)
            update = Update.de_json(data, self.bot)
            self.dp.process_update(update)
        except Exception as e:
            print(e)



import requests,json
from .utils import number_maker,get_random_email,Gate



class gateway(Gate):
    def __init__(self ,**kargs) -> None:
        super().__init__(**kargs)
 
    def pay_link(self,**args):
        amount,callback,order_id = args.get('amount') , args.get('callback'), args.get('order_id')
        if not callback:
            callback = self.callback_maker(order_id)

        mobile,email = args.get('mobile',str(number_maker())) , args.get('email',str(get_random_email()))

        data = {
            'api_key' : self.gate_key ,
            'amount' : amount*10,
            'callback_url' : callback,
            'mobile_number':mobile,
            'comment':email
            }

        headers= {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }
        data =  json.dumps(data)
        response = requests.post('https://ipg.vandar.io/api/v3/send', data =data,headers=headers)

        json_data = json.loads(response.text)
        print(json_data)
        if response.status_code == 200 and str(json_data['status']) == '1':
            token = json_data['token']
            return f"https://ipg.vandar.io/v3/{token}"
        return None            
    
    def verify(self,**args):
        token = args.get('token')
        
        data = {
            'api_key' : self.gate_key,
            'token' : token
        }
        headers= {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }
        data =  json.dumps(data)
        response = requests.post('https://ipg.vandar.io/api/v3/verify', data = data,headers=headers)
        json_data = json.loads(response.text)
        if response.status_code == 200 and str(json_data['status']) == '1':
            return True ,args
        return False  ,args


import ssl
from python_socks.sync import Proxy  as proxy
import certifi
import python_socks

class PROXY:
    def __init__(self,proxy_type:str,ip:str,port:int,username:str=None,password:str=None,timeout=5) -> None:
        '''
            proxy_type:str
            http , socks4 ,socks5
        '''
        if proxy_type not in ['http' , 'socks4' ,'socks5']:
            raise Exception('wrong proxy_type')
        self.proxy_type = proxy_type
        self.ip = ip
        self.port = port
        self.username = username
        self.password = password
        self.timeout = timeout
    def make_proxy(self):
        out = {}
        if self.proxy_type == 'http':
            proxy_type = python_socks.ProxyType.HTTP
        elif self.proxy_type == 'socks4':
            proxy_type = python_socks.ProxyType.SOCKS4
        elif self.proxy_type == 'socks5':
            proxy_type = python_socks.ProxyType.SOCKS5
        out['proxy_type'] = proxy_type
        out['addr'] = self.ip
        out['port'] = int(str(self.port))
        if self.username:out['username'] = self.username
        if self.password:out['password'] = self.password
        return out
    
    def _str_proxy(self):
        out= f"{self.proxy_type}://"
        if self.username and self.password:
            out+= f"{self.username}:{self.password}@"
        out+= f"{self.ip}:{self.port}"
        return out
    
    def check(self):
        str_proxy = self._str_proxy()
        print(f"checking {str_proxy}...")
        try:
            p = proxy.from_url(str_proxy)

            # `connect` returns standard Python socket in blocking mode
            sock = p.connect(dest_host='check-host.net', dest_port=443,timeout=self.timeout)
            sock = ssl.create_default_context(cafile=certifi.where()).wrap_socket(
                sock=sock,
                server_hostname='check-host.net'
            )
            
            request = (
                b'GET /ip HTTP/1.1\r\n'
                b'Host: check-host.net\r\n'
                b'Connection: close\r\n\r\n'
            )
            sock.sendall(request)
            response = sock.recv(4096)
            return True
        except Exception as e:
            print(e)
            return False
        




import requests,json
from zibal.zibal import zibal
from .utils import number_maker,get_random_email,Gate


class gateway(Gate):
    def __init__(self ,**kargs) -> None:
        super().__init__(**kargs)
        if self.test_mode:
            self.gate_key='zibal'
 
    def pay_link(self,**args):
        amount,callback,order_id = args.get('amount') , args.get('callback'), args.get('order_id')
        if not callback:
            callback = self.callback_maker(order_id)

        mobile,email = args.get('mobile',str(number_maker())) , args.get('email',str(get_random_email()))
        zb = zibal(self.gate_key, callback)
        amount *=10 # IRR
        request_to_zibal = zb.request(amount,mobile=mobile,description=email)
        return f"https://gateway.zibal.ir/start/{request_to_zibal['trackId']}"
    
    def verify(self,**args):
        track_id = args.get('trackId')
        callback = self.callback_maker(args.get('order_id'))
        
        zb = zibal(self.gate_key, callback)
        verify_zibal = zb.verify(track_id)
        verify_result = verify_zibal['result']
        if verify_result==100:
            return True ,args
        return False  ,args
    
   


from hcloud import Client
from hcloud.images.client import Image
from hcloud.locations.client import Location
from hcloud.server_types.client import ServerType
from hcloud.networks.client import Network
from hcloud.datacenters.client import Datacenter
from hcloud.servers.client import Server
from hcloud.primary_ips.client import PrimaryIP
from time import time, sleep


class Hetzner:
    def __init__(self, token,**kargs) -> None:
        self.client = Client(token=token,**kargs)
        self.locations = ['fsn1', 'nbg1', 'hel1', 'hil', 'ash']
        self.datacenters = ['fsn1-dc14', 'nbg1-dc3', 'hel1-dc2']
        
        self.image_names = ['ubuntu-22.04']
        self.plans = ['cx11','cpx11','cx21']

    def create_server(self, name='', plan="cx11", image_name='ubuntu-22.04', location="nbg1"):
        '''
        return server and root password
        '''
        if name == '':
            name = f"Server{int(time())}"
        res = self.client.servers.create(
            name=name,
            server_type=ServerType(name=plan),
            image=Image(name=image_name),
            location=Location(name=location)
            
        )
        return res.server, res.root_password

    def delete_server(self, id):
        self.client.servers.delete(Server(id=id))
        return True
    
    def get_all_servers(self):
        return [i.__dict__['data_model'] for i in self.client.servers.get_all()]

    def power_off(self, id):
        self.client.servers.power_off(Server(id=id))
        return True

    def power_on(self, id):
        self.client.servers.power_on(Server(id=id))
        return True

    def reset_password(self, id):
        return self.client.servers.reset_password(Server(id=id)).root_password

    def reboot(self, id):
        self.client.servers.reboot(Server(id=id))
        return True

    def rebuild(self, id, image_name):
        self.client.servers.rebuild(Server(id=id), Image(name=image_name))
        return True

    def get_all_primary_ips(self):
        return [i.__dict__['data_model'] for i in self.client.primary_ips.get_all()]
    
    def get_primary_ips_by_id(self,id):
        inf = self.client.primary_ips.get_by_id(id=id)
        return inf
    
    def get_server_by_id(self,server_id):
        return self.client.servers.get_by_id(server_id).data_model
        
    def create_primary_ips(self,datacenter_name):
        name = f'ip{int(time())}'
        self.client.primary_ips.create('ipv4',Datacenter(name=datacenter_name),name)
        return True
    
    def delete_primary_ips(self, id):
        self.client.primary_ips.delete(PrimaryIP(id))
        return True

    def lock_primary_ips(self, id):
        self.client.primary_ips.change_protection(PrimaryIP(id), True)
        return True

    def unlock_primary_ips(self, id):
        self.client.primary_ips.change_protection(PrimaryIP(id), False)
        return True

    def delete_ipv6_from_server(self,server_id):
        self.power_off(server_id)
        sleep(10)
        server_info = self.client.servers.get_by_id(server_id)
        sleep(2)
        primary_ipv6 = server_info.public_net.primary_ipv6
        if type(primary_ipv6)!=type(None):
            self.client.primary_ips.unassign(primary_ipv6)
            sleep(2)
        self.delete_primary_ips(primary_ipv6.id)
        sleep(5)
        self.power_on(server_id)
        return True

    def change_server_ip(self,server_id,ip_id):
        self.power_off(server_id)
        sleep(10)
        server_info = self.client.servers.get_by_id(server_id)
        old_ip = None
        sleep(2)
        primary_ipv4 = server_info.public_net.primary_ipv4
        if type(primary_ipv4)!=type(None):
            old_ip = server_info.public_net.ipv4.ip
            self.client.primary_ips.unassign(primary_ipv4)
            sleep(2)
        if ip_id==0:
            new_ip_name= f'ip{int(time())}'
            primary_ip = self.client.primary_ips.create('ipv4',datacenter=server_info.datacenter,name=new_ip_name).primary_ip
        else:primary_ip = self.get_primary_ips_by_id(ip_id)
        sleep(5)
        self.client.primary_ips.assign(primary_ip,server_id)
        sleep(5)
        self.power_on(server_id)
        return old_ip
    
    def assign_to_server(self,ip_id,server_id):
        self.power_off(server_id)
        sleep(10)
        server_info = self.client.servers.get_by_id(server_id)
        sleep(2)
        primary_ipv4 = server_info.public_net.primary_ipv4
        if type(primary_ipv4)!=type(None):
            self.client.primary_ips.unassign(primary_ipv4)
            sleep(2)
        self.client.primary_ips.assign(PrimaryIP(id=ip_id),server_id)
        sleep(5)
        self.power_on(server_id)
        return True
        
    def unassign_from_server(self,ip_id):
        info = self.get_primary_ips_by_id(ip_id)
        assing_id = info.assignee_id
        if assing_id!=None:
            server_id = info.assignee_id
            self.power_off(server_id)
            sleep(10)
            self.client.primary_ips.unassign(PrimaryIP(id=ip_id))
        return True
    
    def batch_ip4_creator(self,datacenter_name,count):
        success=0
        for i in range(count):
            try:
                new_ip_name= f'ip{int(time())}'
                self.client.primary_ips.create('ipv4',datacenter=Datacenter(name=datacenter_name),name=new_ip_name)
                success+=1
            except:pass
            sleep(10)
        return success





    
if __name__ == "__main__":
    token = '9Ii0sFqfxOZX3K4mGjIpGMxDHEJfyFjJ9rRGySNm7hRyJaJwK3ncbOwh9hx0FNLb'
    h = Hetzner(token)
    # id=35188187
    
    # for i in h.get_all_servers():
    #     print(f"{i.datacenter.__dict__}")
    #     break
    # h.change_server_ip(id)
    # Server().status

    # print(h.create_server())
    # sid = 35188187
    # print(h.power_on(sid))
    # print(h.reboot(sid))
    # print(h.reset_password(sid))
    # print(h.rebuild(sid,h.image_names[0]))
    # h.delete_primary_ips(id)
    
    # for i in h.get_all_primary_ips():
    #     print(i)

    # o = h.delete_server(35187409)
    # print(o)



from tkinter import ttk
from tkinter import *
from functools import partial
from .utils import *
from starco.pkl import Pkl

class GUI:
    def __init__(
            self, title: str,
              tabs={'start': {'rows': 5}},
                tabs_padding=15,
                width=350,
                height=500
                ,full_screen=False,
                bg_color ='#000',
                fg_color ='#fff',
                **kargs
                ) -> None:
        '''
            pkl_path
        '''
        self.vars = {}
        self.root = Tk()
        self.root.geometry(f"{width}x{height}")
        if full_screen:
            FullScreenApp(self.root)
        self.bg_color = bg_color
        self.fg_color = fg_color
        self.root.update()
        self.root.title(title)
        self.x_y_info = {}
        self.pkl = Pkl(kargs.get('pkl_path','setting'))
        self.tabs_padding = tabs_padding
        self.tabs = tabs
        self.init_tabs()
        self.set_styles()

    def set_styles(self):
        style = ttk.Style(self.root)
        style.theme_use('alt')
        styles = {}
        styles['frame'] = ttk.Style()
        styles['frame'].configure('new.TFrame', background=self.bg_color,foreground=self.fg_color)
        styles['checkbutton'] = ttk.Style()
        styles['checkbutton'].configure('new.TCheckbutton', background=self.bg_color,foreground=self.fg_color)
        styles['label'] = ttk.Style()
        styles['label'].configure("new.TLabel", background=self.bg_color,foreground=self.fg_color)
        styles['treeview'] = ttk.Style()
        styles['treeview'].configure('Treeview', rowheight=40)

        # ('aqua', 'step', 'clam', 'alt', 'default', 'classic')
    
    def init_tabs(self):

        tabControl = ttk.Notebook(self.root, padding=self.tabs_padding)
        for tab in self.tabs.keys():
            self.vars[tab] = ttk.Frame(tabControl, padding=self.tabs_padding, style='new.TFrame')
            tabControl.add(self.vars[tab], text=tab.title())

        tabControl.pack(expand=1, fill="both")

    def callback(self, sv, *args):
        self.pkl.pkl(str(sv), sv.get())

    def label_txt(self, str_in):
        return str_in.replace("_", " ").title()+" : "

    def get_row(self, name: str):
        row = self.x_y_info.get(name, 0)
        self.x_y_info[name] = row+1
        return row

    def get_rely(self, name: str):
        row = self.get_row(name)
        step = 1/self.tabs[name]['rows']
        return row*(step)

    def add_label_input(self, tab, name: str, label_to_input_ratio=0.5, end_label: str = None, label: str = None, savable: bool = True, empty_return: str = '', margin: float = 0, input_justify=LEFT):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        if not label:
            label = self.label_txt(name)
        item_w = label_to_input_ratio * (1-2*margin)
        self.vars[name] = StringVar(value=self.pkl.pkl(
            name, empty_return=empty_return), name=name)

        a = ttk.Label(root, text=label, style='new.TLabel')
        indent = margin
        a.place(relx=indent, rely=rely, relwidth=item_w)

        if savable:
            self.vars[name].trace("w", lambda name, index,
                                  mode, sv=self.vars[name]: self.callback(sv))
        relwidth = (1-label_to_input_ratio) * (1-2*margin)
        if end_label != None:
            relwidth = (1-2*label_to_input_ratio) * (1-2*margin)

        b = ttk.Entry(
            root, textvariable=self.vars[name], justify=input_justify)
        indent += item_w
        b.place(relx=indent, rely=rely, relwidth=relwidth)

        if end_label != None:
            indent += relwidth+0.01
            c = ttk.Label(root, text=end_label, style='new.TLabel')
            c.place(relx=indent, rely=rely, relwidth=item_w)

    def add_label(self, tab, name: str, label: str = None, margin: float = 0, input_justify=CENTER):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        if not label:
            label = name
        item_w = (1-2*margin)

        a = ttk.Label(root, text=label, style='new.TLabel',justify=input_justify)
        indent = margin
        a.place(relx=indent, rely=rely, relwidth=item_w)

    def add_combobox(self, tab, name: str, values: list, label_to_input_ratio=0.5, end_label: str = None, label: str = None, savable: bool = True, empty_return: str = '', margin: float = 0):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        if not label:
            label = self.label_txt(name)
        item_w = label_to_input_ratio * (1-2*margin)
        self.vars[name] = StringVar(value=self.pkl.pkl(
            name, empty_return=empty_return), name=name)

        a = ttk.Label(root, text=label, style='new.TLabel')
        indent = margin
        a.place(relx=indent, rely=rely, relwidth=item_w)

        relwidth = (1-label_to_input_ratio) * (1-2*margin)
        if end_label != None:
            relwidth = (1-2*label_to_input_ratio) * (1-2*margin)

        b = ttk.Combobox(
            root, justify=CENTER, textvariable=self.vars[name], values=values, state='readonly')
        if savable:
            b.bind("<<ComboboxSelected>>", partial(
                self.callback, self.vars[name]))
        indent += item_w
        b.place(relx=indent, rely=rely, relwidth=relwidth)

        if end_label != None:
            indent += relwidth+0.01
            c = ttk.Label(root, text=end_label, style='new.TLabel')
            c.place(relx=indent, rely=rely, relwidth=item_w)

    def add_chekbox(self, tab, name: str, label_to_input_ratio=0.5, label: str = None, empty_return: str = '', margin: float = 0):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        if not label:
            label = self.label_txt(name)
        item_w = label_to_input_ratio * (1-2*margin)
        self.vars[name] = StringVar(value=self.pkl.pkl(
            name, empty_return=empty_return), name=name)

        a = ttk.Label(root, text=label, style='new.TLabel')
        indent = margin
        a.place(relx=indent, rely=rely, relwidth=item_w)
        relwidth = (1-label_to_input_ratio) * (1-2*margin)
        b = Checkbutton(root, command=partial(self.callback, self.vars[name]), background=self.bg_color, activebackground=self.bg_color,
                        highlightcolor=self.bg_color, highlightbackground=self.bg_color, borderwidth=None, variable=self.vars[name], onvalue=1, offvalue=0)
        indent += item_w
        b.place(relx=indent, rely=rely, relwidth=relwidth)

    def add_button(self, tab, name: str, label: str = None, bg: str = 'green', command=None, margin: float = 0):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        indent = margin
        relwidth = 1-2*indent
        # self.vars[name] = StringVar(value=self.pkl.pkl(name,empty_return=empty_return),name=name)
        btn =  Button(root, text=label, bg=bg)
        if type(command)!=type(None):
            btn.config(command=partial(command,self.vars))
        self.vars[name] =btn
        self.vars[name].place(relx=indent, rely=rely, relwidth=relwidth)

    def add_frame(self,tab, name: str,fill_raw=1,relwidth=1,relx=0):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        for _ in range(fill_raw-1):
            self.get_row(tab)
        rely2= self.get_rely(tab)
        
        self.vars[name] = Frame(root)
        self.vars[name].place(relx=relx,rely=rely,relwidth=relwidth,relheight=abs(rely2-rely))
      
    def add_label_text(self, tab, name: str, label_to_input_ratio=0.5, label: str = None, savable: bool = True, empty_return: str = '', margin: float = 0,height=2,state=DISABLED):
        root = self.vars[tab]
        rely = self.get_rely(tab)
        if not label:
            label = self.label_txt(name)
        item_w = label_to_input_ratio * (1-2*margin)
        a = ttk.Label(root, text=label, style='new.TLabel')
        indent = margin
        a.place(relx=indent, rely=rely, relwidth=item_w)

        relwidth = (1-label_to_input_ratio) * (1-2*margin)
        

        self.vars[name] = Text(root ,height= height)
        self.vars[name].config(state=state)
        indent += item_w
        self.vars[name].place(relx=indent, rely=rely, relwidth=relwidth)
        
    def add_tabel(self, tab, name: str,columns_dict:dict,inputs_show='v',remoev_duplicate=None):
        '''
        columns_dict:
        w : columns width
        justify':RIGHT
        filter : list of finction
        options: auto complete options
        defulte_text: string
        action : function
        is_timestamp: bool or str format filter_column=sum , selection
        
        '''
        root = self.vars[tab]
        if remoev_duplicate!=None:
            remoev_duplicate=name+remoev_duplicate
        table =Custom_Table(name, root, self.vars, self.pkl.pkl,columns_dict,inputs_show=inputs_show,remoev_duplicate=remoev_duplicate)
        table.show_data()

    def close(self,func=None,*args,**fun_kargs):
        if type(func)!=type(None):
            func(**fun_kargs)
        self.root.destroy()
    
    def run(self):
        self.root.mainloop()


if __name__ == "__main__":
    pass


from ..base import Base
from ..classes import Conversation
from .enum import Pack,Node
from .filteres import match_btn,match_text
from telegram.ext import Filters
from starco.tlg.bot.util.enum import *
from starco.tlg.bot.classes import Conversation
from starco.utils import path_maker
import telegram
from telegram.ext import Filters
from starco.tlg.app.utils import get_number,session_number
from starco.tlg.app import TlgApp
import threading,re,time
from functools import partial
import shutil
from starco.tlg.bot.util.filteres import *
from starco.tlg.bot.util.functions import copy_ready_session
from starco.utils import chunks

def Start(self:Conversation):
    
    def act(self:Conversation,*args):
        try:
            self.send(msg = 'start_pm',btns = self.menu_keys,get_text_setting=True)
        except Exception as e:print(e)
        return -1

    p = Pack()
    p.name = 'start'

    e1 = Node()
    e1.command = 'start'
    e1.command_filters = match_text('/start')
    e1.callback=act

    e2=Node()
    e2.pattern = self.check_inline_keyboards('/start')
    e2.callback = act
    p.entry=[e1,e2]
    return p

def ForceJoinPack(self:Conversation,**kargs):
    p = Pack()
    get_text_setting =kargs.get('get_text_setting',False)
    p.name="force_join_pack"
    p.db_config = {'force_join': {'id': 0, 'title': '', 'channel_id': '', 'link': '','status':0}}
    
    e=Node()
    e.filters = ~ForceJoinFilter(self)
    def act(self:Conversation,*a):
        try:
            start_bot = self.splited_query_data()[0]
            if start_bot == 'start_bot':
                self.alert(self.text('join_alert_pm'))
                return -1
        except:
            pass
        btns = {}
        for i in self.db.do('force_join',condition=f"status={CONFIRMED}"):
            link = i['link']
            btns[i['title']] = link
            txt = self.get_text()
            if '/start' in txt:
                btns['start_bot'] =txt 
            else:btns['start_bot'] ='/start'
        self.send(msg='join_pm', btns = btns,col=1,get_text_setting=get_text_setting,close=False)
        return -1
    e.callback =act

    e1=Node()
    e1.pattern = self.force_join_inline_filter()
    e1.callback=act

    ch = Node()
    ch.pattern = self.check_inline_keyboards('start_bot')
    def act(self:Conversation,*a):
        do_force_join_del_pm(self)
        return -1

    ch.callback=act
    
    p.entry=[e,e1,ch]
    return p

def do_force_join_del_pm(self:Conversation):
    try:
        self.splited_query_data()[0]
        self.delete_message(self.get_msg_id())
    except:pass

def SharePhone(self:Conversation,**kargs):
    p = Pack()
    p.name = 'share_phone'
    get_text_setting =kargs.get('get_text_setting',False)

    e1=Node()
    e1.filters = SharePhoneFilter(self)
    def show_pm(self,*a):
        self.send('share_phone', [['share_phone']], share_phone=True,get_text_setting=get_text_setting)
        return -1
    e1.callback = show_pm

    e=Node()
    e.filters = Filters.contact
    def act(self:Conversation,*args):
        contact = self.update.effective_message.contact
        phone = contact.phone_number
        self.db.do('users', {'id': self.id, 'phone': int(phone)}, condition=f"id={self.id}")
        self.user_info=self.get_user_info_from_db([])
        self.send_message('phone_shared', self.menu_keys)
        return -1
    e.callback = act
    p.entry=[e1,e]
    return p
            
def SelectLanguagePack(self:Conversation,**kargs):
    get_text_setting =kargs.get('get_text_setting',False)
    # next_check= kargs.get('next_check')
    p = Pack()
    p.name = 'select_language'
    e=Node()
    e.filters =SelectLanguageFilter(self)
    def act(self:Conversation,*a):
        btn = {k:f"{v}:select_language" for k,v in self.languages.items()}
        self.send(msg='Select_your_Language', btns= btn,get_text_setting=get_text_setting,close=False)
        return -1
    e.callback =act

    e2=Node()
    e2.command ='Language'
    def act(self:Conversation,*a):
        btn = {k:f"{v}:select_language" for k,v in self.languages.items()}
        self.send(msg='Select_your_Language', btns= btn,get_text_setting=get_text_setting,close=False)
        # if type(next_check)!=type(None):
        #     next_check(self)
        return -1
    e2.callback =act

    e1 = Node()
    e1.pattern = self.check_inline_keyboards('select_language')
    def act(self:Conversation,*args):
        lang_id = int(self.splited_query_data()[1])
        self.db.do('users', {'id': self.id, 'language': lang_id}, condition=f"id={self.id}")
        self.delete_message(self.get_msg_id())
        self.user_info=self.get_user_info_from_db([])
        self.send('language_set', self.menu_keys,get_text_setting=get_text_setting)
        
        return -1
    e1.callback = act
    p.entry=[e,e1,e2]
    return p
            
def Referral(self:Conversation):
    p = Pack()
    p.name = 'Referral'
    p.db_config={'users':{'presenter':0}}

    e1 = Node()
    e1.btn = self.menu_keys
    e1.filters = match_btn('referral',self)
    
    def referral_text(self:Base,*args):
        msg=self.text('referral_text',slash=True)+'\n\n'
        msg+=f'https://t.me/{self.super_self.bot_username}?start=ref{self.id}\n\n'
        msg+=f"{self.text('invited')} : {len(self.db.do('users', condition=f'presenter={self.id}'))}"

        self.send(msg= msg,btns=self.menu_keys,translat=False)
        return -1
    e1.callback =referral_text

    e2 = Node()
    e2.btn = self.menu_keys
    e2.command = 'start'
    e2.command_filters=Filters.regex('/start ref\d+')
    def referral_action(self,*args):
        try:
            presenter = int(self.get_text().split(' ')[-1].replace('ref',''))
        except:
            try:
                presenter = int(self.splited_query_data()[1].split(' ')[-1].replace('ref',''))
            except:
                presenter=0
        try:
            submited_presenter = int(self.user('presenter'))
        except:
            submited_presenter = 0
        if submited_presenter==0 and presenter!=self.id:
            self.db.do('users', {'presenter': presenter},condition=f"id={self.id}")
            self.send('new_subset_text', chat_id=presenter)
        self.send('start_pm', self.menu_keys)
        return -1
    
    e2.callback=referral_action

    e3 = Node()
    e3.btn = self.menu_keys
    e3.pattern = self.check_inline_keyboards('/start ref',regex=True)
    e3.callback=referral_action


    p.entry=[e1,e2,e3]
    return p

def LoginTlgAccount(self:Conversation):
    p = Pack()
    p.name = 'login_tlg_account'
    p.db_config={'tlg_accounts':{'id':0,'u_id':0,'status':0,'spam_check':0,'category':'','fastremove':0,'limit_status':'','limit_time':0,'last_check':0,'active_sessions':0,'time':0}}
    e=Node()
    e.filters = Filters.regex('\+\d{9,12}') | Filters.contact
    def act(self: Conversation, *args):
        self.userdata('tlg',delete=True)
        if Filters.contact(self.update):
            contact = self.update.effective_message.contact
            txt = contact.phone_number
        else:
            txt = self.get_text()
        last_msg_id = self.get_msg_id() 
        
        number = get_number(txt)
        if len(txt) > 8 and number and number > 0:
            db_ninfo = self.db.do('tlg_accounts', condition=f'id={number}')
            db_ninfo = [i for i in db_ninfo if i['status'] in [CONFIRMED,WAITING]]
            if db_ninfo:
                self.send('submit_before',self.menu_keys)
                return -1
            else:
                path_number=session_number(number)
                
                self.send('send_code_note',[[self.back_menu_key]],reply_to_message_id=last_msg_id)
                tlg = TlgApp(path_number, auto_connect=False)
                tlg.d['pm_msg_id'] =last_msg_id
                self.userdata('tlg', tlg)
                def action(self,tlg:TlgApp):
                    try:
                        tlg.connect()
                        out = tlg.do_login(True)
                        if out != '':
                            self.send(out,translat=False,reply_to_message_id=last_msg_id)
                    except:
                        self.send('try_again',reply_to_message_id=last_msg_id)
                threading.Thread(target=partial(action,self,tlg)).start()
                return self.stat_key('login_tlg_account', 0)

        else:
            self.send(self.text('bad_number',slash=True)+'\n'+self.text('menu_text',slash=True),translat=False,reply_to_message_id=last_msg_id)
            return -1
    e.callback = act


    def add_pendding(self: Conversation):
        tlg: TlgApp = self.userdata('tlg')
        tlg.ative_last_seen()
        tlg.disable_show_phone_number()
        tlg.disconnect()
        time.sleep(1)
        oid = get_number(tlg.number)
        cfg = {
            'id': oid,
            'u_id': self.id,
            'status': CONFIRMED,
            'category':'manual',
            'time': self.time()}
        self.db.do('tlg_accounts', cfg,condition=f"id={oid}")
        self.send(self.text('submited')+f'\n/tlg_{oid}',reply_to_message_id=tlg.d.get('pm_msg_id'))
        
    s2 = Node()
    s2.filters = Filters.text

    def act(self: Conversation, *args):
        code = self.get_text()
        if 'Login code' in code:
            find = re.findall('Login code: (\d{5}).',code)
            if find:
                code = find[0]
        tlg: TlgApp = self.userdata('tlg')
        if type(tlg)==type(None):
            self.send('try_again')
            return -1

        if len(code) >= 5 and type(tlg)!=type(None) and tlg.d.get('step')==2:
            code = int(re.search(r'\d+', str(code)).group())
            out = tlg.do_login(code=code)
            if out == '':
                add_pendding(self)
                return -1
            elif out == 'need_pass2':
                self.send(self.text(out,slash=True)+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
                return self.stat_key('login_tlg_account', 1)
            else:
                number = tlg.number
                msg = self.text(out, slash=True)+f'\nnumber:{number}\n\n'
                msg += self.text('send_code', slash=True)

                self.send(msg+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
                return self.stat_key('login_tlg_account', 0)

        else:
            self.send(self.text('send_code',slash=True)+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
            return self.stat_key('login_tlg_account', 0)
    s2.callback = act

    s3 = Node()
    s3.filters = Filters.text

    def act(self: Conversation, *args):
        p2fa = self.get_text()
        
        tlg: TlgApp = self.userdata('tlg')
        out = tlg.do_login(p2fa=p2fa)
        if out == '':
            add_pendding(self)
            return -1
        else:
            msg = self.text(out, slash=True)+'\n'
            msg += self.text('need_pass2', slash=True)

            self.send(msg+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
            return self.stat_key('login_tlg_account', 1)
    s3.callback = act

    p.entry=[e]
    p.states=[[s2], [s3]]

    return p

def ReloginTlgAccount(self:Conversation):
    p = Pack()
    p.name = 'relogin_tlg_account'

    e=Node()
    e.pattern = self.check_inline_keyboards('relogin')
    def act(self: Conversation, *args):
        last_msg_id = self.get_msg_id()
        number = get_number(self.splited_query_data()[-1])
        if number > 0:
            db_ninfo = self.db.do('tlg_accounts', condition=f'id={number}')
            db_ninfo = [i for i in db_ninfo if i['status'] in [CONFIRMED]]
            if not db_ninfo:
                self.send('session_not_active')
                return -1
            else:
                path_number=session_number(number)
                password_2fa= TlgApp(path_number,auto_connect=False).js('password_2fa')
                self.send(f"pass2:`{password_2fa}`",translat=False,parse_mode=telegram.ParseMode.MARKDOWN)
                tlg = TlgApp(path_number, auto_connect=False,path_list =['relogin',number], js_path_list =['relogin',number] )
                self.send('send_code_note',reply_to_message_id=last_msg_id)
                tlg.d['pm_msg_id'] =last_msg_id
                self.userdata('tlg', tlg)
                def action(self,tlg:TlgApp):
                    try:
                        tlg.connect()
                        out = tlg.do_login(True)
                        time.sleep(10)
                        otlg = TlgApp(tlg.number)
                        logcode = otlg.get_telegram_code()
                        otlg.disconnect()
                        self.send(f"`{logcode}`",translat=False,parse_mode=telegram.ParseMode.MARKDOWN)
                        # if str(len(logcode)>=5):
                        #     login_proccess = tlg.do_login(code=logcode)
                        #     print(login_proccess)
                        #     if login_proccess=='':
                        #         add_pendding(self)
                        #         return -1
                        #     elif login_proccess=='need_pass2':
                        #         password_2fa = otlg.js('password_2fa')
                        #         res_out=tlg.do_login(p2fa=password_2fa)
                        #         print(res_out)
                        #         if res_out=='':
                        #             add_pendding(self)
                        #             return -1
                        if out != '':
                            self.send(out,translat=False,reply_to_message_id=last_msg_id)
                    except:
                        self.send('try_again',reply_to_message_id=last_msg_id)
                threading.Thread(target=partial(action,self,tlg)).start()
                return self.stat_key('relogin_tlg_account', 0)

        else:
            self.send(self.text('bad_number',slash=True)+'\n'+self.text('menu_text',slash=True),translat=False,reply_to_message_id=last_msg_id)
            return -1
    e.callback = act


    def add_pendding(self: Conversation):
        tlg: TlgApp = self.userdata('tlg')
        tlg.ative_last_seen()
        tlg.disable_show_phone_number()
        tlg.disconnect()
        time.sleep(1)
        oid = get_number(tlg.number)
        self.db.do('tlg_accounts',{'category':'relogin'},condition=f"id={oid}")
        temp_tlg = TlgApp(oid,auto_connect=False)
        shutil.move(tlg.path,temp_tlg.path)
        shutil.move(tlg.json_path,temp_tlg.json_path)
        self.send(self.text('done')+f'\n/tlg_{oid}',reply_to_message_id=tlg.d.get('pm_msg_id'))
        
    s2 = Node()
    s2.filters = Filters.text

    def act(self: Conversation, *args):
        code = self.get_text()
        if 'Login code' in code:
            find = re.findall('Login code: (\d{5}).',code)
            if find:
                code = find[0]
        tlg: TlgApp = self.userdata('tlg')
        if type(tlg)==type(None):
            self.send('try_again')
            return -1

        if len(code) >= 5 and type(tlg)!=type(None) and tlg.d.get('step')==2:
            code = int(re.search(r'\d+', str(code)).group())
            out = tlg.do_login(code=code)
            if out == '':
                add_pendding(self)
                return -1
            elif out == 'need_pass2':
                self.send(self.text(out,slash=True)+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
                return self.stat_key('relogin_tlg_account', 1)
            else:
                number = tlg.number
                msg = self.text(out, slash=True)+f'\nnumber:{number}\n\n'
                msg += self.text('send_code', slash=True)

                self.send(msg+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
                return self.stat_key('relogin_tlg_account', 0)

        else:
            self.send(self.text('send_code',slash=True)+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
            return self.stat_key('relogin_tlg_account', 0)
    s2.callback = act

    s3 = Node()
    s3.filters = Filters.text

    def act(self: Conversation, *args):
        p2fa = self.get_text()
        
        tlg: TlgApp = self.userdata('tlg')
        out = tlg.do_login(p2fa=p2fa)
        if out == '':
            add_pendding(self)
            return -1
        else:
            msg = self.text(out, slash=True)+'\n'
            msg += self.text('need_pass2', slash=True)

            self.send(msg+f"\n/cancel",translat=False,reply_to_message_id=tlg.d.get('pm_msg_id'))
            return self.stat_key('relogin_tlg_account', 1)
    s3.callback = act

    p.entry=[e]
    p.states=[[s2], [s3]]

    return p

def CopySession(self:Conversation):
    p=Pack()
    p.name = 'copy_session'
    e =Node()
    e.filters=FileFormat('zip')
    def act(self:Conversation,*args):
        file = self.update.effective_message.effective_attachment
        file_name= file.file_name
        if '.zip' in file_name:
            zip_path = path_maker(['temp'],'.')
            info = self.bot.get_file(file.file_id)
            zip_path=info.download(zip_path+f"/{info.file_path.split('/')[-1]}")
            saved_before = [i['id'] for i in self.db.do('tlg_accounts', condition=f'status={CONFIRMED}')]
            numbers = copy_ready_session(zip_path,saved_before)
            for number in numbers:
                try:
                    tlg = TlgApp(number,disable_proxy=True,auto_connect=False)
                    tlg.remove_proxy()
                    tlg.connect()
                    status =tlg.get_status()
                    if status:
                        tlg.ative_last_seen()
                        tlg.disable_show_phone_number()
                        oid = get_number(tlg.number_int)
                        cfg = {
                            'id': oid,
                            'u_id': self.id,
                            'status': CONFIRMED,
                            'category':'copied',
                            'time': self.time()
                            }
                        self.db.do('tlg_accounts', cfg,condition=f"id={oid}")
                        self.send(self.text('submited')+f'\n/tlg_{oid}',disable_notification=True)
                    else:self.send(f"{number} "+self.text('session_not_active',slash=True),disable_notification=True)
                except:pass
                finally:
                    try:tlg.disconnect()
                    except:pass
        return -1
    
    e.callback =act
    p.entry=[e]

    return p

def ToggleSetting(self:Conversation,**kargs):
    p=Pack()
    key = kargs.get('key','toggle_setting')
    p.name = key
    def setting_msg_btn(self):
        setting = self.db.do('setting',condition=f"subset='{key}'")
        msg=''
        btn={}
        for i in setting:
            msg+=f"{self.text(i['key'],slash=True)} : {self.text(i['value'])}\n"
            btn[i['key']]=key
        return msg,btn

    e=Node()
    e.filters = match_btn(key,self)
    def act(self:Conversation,*args):
        msg,btn = setting_msg_btn(self)
        self.send(msg,btn,translat=False)
        return self.stat_key(key,0)
    e.callback=act

    s=Node()
    s.pattern = self.check_inline_keyboards(key)
    def act(self:Conversation,*args):
        k = self.splited_query_data()[0]
        res = self.toggle('setting','value',k,'key',['0','1'])
        self.alert(self.text(res))
        msg,btn = setting_msg_btn(self)
        self.edit_message_text(msg,self.get_msg_id(),btn,t=False)
        return self.stat_key(key,0)
    s.callback=act

    p.entry=[e]
    p.states=[[s]]

    return p

def Status(self:Conversation,**kargs):
    p=Pack()
    subset = kargs.get('key','toggle_setting')
    column = kargs.get('column','subset')

    msg = kargs.get('msg','click_option')
    table = kargs.get('table','setting')
    value = kargs.get('value','value')
    key = kargs.get('target','key')
    p.name = subset
    def setting_msg_btn(self):
        setting = self.db.do('setting',condition=f"{column}='{subset}'")
        btn=[]
        for item in setting:
            sbtn=[
                telegram.InlineKeyboardButton(self.text(item[key]),callback_data=f'{item[key]}:{subset}slbl'),
                telegram.InlineKeyboardButton(self.text(item[value]),callback_data=f'{item[key]}:{subset}set_stat'),
            ]
            btn+=[sbtn]
        btn = telegram.InlineKeyboardMarkup(btn)
        return msg,btn
    e=Node()
    e.filters = match_btn(subset,self)
    def act(self:Conversation,*args):
        msg,btn = setting_msg_btn(self)
        self.send(msg,btn)
        return -1
    e.callback=act
    ###############################
    s=Node()
    s.pattern =self.check_inline_keyboards(subset+'slbl')
    def act(self:Conversation,*a):
        name = self.splited_query_data()[0]
        self.alert(self.text(name))
        return -1
    s.callback=act
    ###############################
    s1=Node()
    s1.pattern =self.check_inline_keyboards(subset+'set_stat')
    def act(self:Conversation,*a):
        name = self.splited_query_data()[0]
        res = self.toggle(table,value,name,cond_col=key,switch=['0','1'])
        _,btn = setting_msg_btn(self)
        self.edit_message_reply_markup(btn,self.get_msg_id(),col=4)
        return -1
    s1.callback=act
    p.entry=[e,s,s1]
    return p

def Setting(self:Conversation,**kargs):
    p=Pack()
    key = kargs.get('key','public_setting')
    p.name = key

    def setting_msg_btn(self):
        setting = self.db.do('setting',condition=f"subset='{key}'")
        msg=''
        btn=[]
        for i in setting:
            if i.get('hide')!=1:
                msg+=f"{self.text(i['key'],slash=True)} : \n{i['value']} {self.text(i['unit'],slash=True)}\n---\n"
            btn+=[i['key']]
        if msg=='':msg=self.text('select_option',slash=True)
        return msg,btn

    e=Node()
    e.filters = match_btn(key,self)
    def act(self:Conversation,*args):
        msg,btn = setting_msg_btn(self)
        self.send(msg,chunks(btn,2)+[[self.back_menu_key]],translat=False)
        return self.stat_key(key,0)
    e.callback=act

    s=Node()
    s.filters = Filters.text
    def act(self:Conversation,*args):
        msg,btn = setting_msg_btn(self)
        k = self.get_btn_key_as_text(self.get_text())
        if k in btn:
            msg='enter_new_value'
            self.userdata('set_key',k)
            self.send(msg,[[self.back_menu_key]])
            return self.stat_key(key,1)
        else:
            btn+=[self.back_menu_key]
            self.send(msg,chunks(btn,2),col=2,translat=False)
            return self.stat_key(key,0)
    s.callback=act


    s1=Node()
    s1.filters = Filters.text
    def act(self:Conversation,*args):
        k = self.userdata('set_key')
        if self.set_setting(k,self.get_text()):
            self.send('updated')
        else:
            self.send(self.text('bad_input',slash=True)+'\n'+self.text('enter_new_value',slash=True))
            return self.stat_key(key,1)

        msg,btn = setting_msg_btn(self)
        self.send(msg,chunks(btn,2)+[[self.back_menu_key]],col=2,translat=False)
        return self.stat_key(key,0)
    s1.callback=act

    p.entry=[e]
    p.states=[[s],[s1]]

    return p
    
def SettingPro(self:Conversation,**kargs):
    p=Pack()
    key = kargs.get('key','setting_pro')
    p.name = key

    def setting_msg_btn(self):
        setting = self.db.do('setting',condition=f"subset='{key}'")
        msg=''
        btn=[]
        toggle_btn=[]
        toggle_msg='click_option'
        for i in setting:
            if i.get('hide')!=1:
                if i.get('toggle')==1:
                    sbtn=[                        
                        telegram.InlineKeyboardButton(self.text(i['key']),callback_data=f"{i['key']}:{key}slbl"),
                        telegram.InlineKeyboardButton(self.text(i['value']),callback_data=f"{i['key']}:{key}set_stat"),
                    ]
                    toggle_btn+=[sbtn]
                else:
                    msg+=f"{self.text(i['key'],slash=True)} : \n{i['value']} {self.text(i['unit'],slash=True)}\n---\n"
                    btn+=[i['key']]
            
        if msg=='':msg=self.text('select_option',slash=True)
        if toggle_btn:
            toggle_btn = telegram.InlineKeyboardMarkup(toggle_btn)

        return msg,btn,toggle_msg,toggle_btn

    e=Node()
    e.filters = match_btn(key,self)
    def act(self:Conversation,*args):
        msg,btn,toggle_msg,toggle_btn = setting_msg_btn(self)
        if toggle_btn:
            self.send(toggle_msg,toggle_btn,translat=False)
        if btn:
            self.send(msg,chunks(btn,2)+[[self.back_menu_key]],translat=False)
        return self.stat_key(key,0)
    e.callback=act

    ###############################
    ss=Node()
    ss.pattern =self.check_inline_keyboards(key+'slbl')
    def act1(self:Conversation,*a):
        name = self.splited_query_data()[0]
        self.alert(self.text(name))
        return self.stat_key(key,0)
    ss.callback=act1
    ###############################
    ss1=Node()
    ss1.pattern =self.check_inline_keyboards(key+'set_stat')
    def act2(self:Conversation,*a):
        name = self.splited_query_data()[0]
        res = self.toggle('setting','value',name,cond_col='key',switch=['0','1'])
        _,_,_,toggle_btn = setting_msg_btn(self)
        self.edit_message_reply_markup(toggle_btn,self.get_msg_id(),col=4)
        return self.stat_key(key,0)
    ss1.callback=act2


    s=Node()
    s.filters = Filters.text
    def act(self:Conversation,*args):
        msg,btn,_,_ = setting_msg_btn(self)
        k = self.get_btn_key_as_text(self.get_text())
        if k in btn:
            msg='enter_new_value'
            self.userdata('set_key',k)
            self.send(msg,[[self.back_menu_key]])
            return self.stat_key(key,1)
        else:
            self.send(msg,chunks(btn,2)+[[self.back_menu_key]],col=2,translat=False)
            return self.stat_key(key,0)
    s.callback=act


    s1=Node()
    s1.filters = Filters.text
    def act(self:Conversation,*args):
        k = self.userdata('set_key')
        if self.set_setting(k,self.get_text()):
            self.send('updated')
        else:
            self.send(self.text('bad_input',slash=True)+'\n'+self.text('enter_new_value',slash=True))
            return self.stat_key(key,1)

        msg,btn,_,_ = setting_msg_btn(self)
        self.send(msg,chunks(btn,2)+[[self.back_menu_key]],col=2,translat=False)
        return self.stat_key(key,0)
    s1.callback=act

    p.entry=[e]
    p.states=[[s,ss,ss1],[s1]]

    return p
   
def Products(self:Conversation,**kargs):
    p=Pack()
    row = kargs.get('row',5)
    p.name='products'
    p.db_config = {
        'products':{
            'id':0,
            'title':'',
            'price':0.,
            'status':0,
            'time':0,
        }
        }
    e = Node()
    e.filters = match_btn('products', self)
    def act(self:Conversation,*args):
        products = self.db.do('products')
        main_btn = [['add_product'],[self.back_menu_key]]
        if products:
            self.send('products_list',btns=main_btn)
            msg =[]
            for i in products:
                msg+=[f"{i['title']}\n/p_{i['id']}\n\n"]
            msg,btn = self.pagination_maker(msg,row)
            self.send(msg=msg,btns=btn,t=False)

        else:
            self.send('no_item',main_btn)
        return -1
    e.callback = act
    p.entry=[e]
    return p

def Orders(self:Conversation,**kargs):
    p=Pack()
    row = kargs.get('row',5)
    p.name='orders'
    p.db_config = {
        'orders':{
            'id':0,
            'pid':'',
            'uid':'',
            'price':0.,
            'status':0,
            'time':0,
        }
        }
    e = Node()
    e.filters = match_btn('orders', self)
    def act(self:Conversation,*args):
        orders = self.db.do('orders')
        main_btn = self.menu_keys
        if orders:
            self.send(msg,btns=main_btn,translat=False)
            msg =[]
            for i in orders:
                msg+=[f"/o_{i['id']}\n"]
            msg,btn = self.pagination_maker(msg,row)
            self.send(msg=msg,btns=btn,t=False)
        else:
            self.send('no_item',main_btn)
        return -1
    e.callback = act
    p.entry=[e]
    return p

from starco.tlg.bot.util.filteres import match_btn
from starco.tlg.bot.util.enum import *

def Multi(self:Conversation,name,items:list,check_sign='✅',table='setting',cond='key=',target='value',target_type=str,close=True):
    items = dict(zip(range(len(items)),items))
    p =Pack()
    p.name = name
    e=Node()
    e.filters = match_btn(name,self)
    def act(self:Conversation,*a):
        msg= f'{name}_txt'
        value = int(str(self.db.do(table,condition=cond)[0][target]))
        if value==None:value=0
        btn = {}
        for k,v in items.items():
            sign=''
            if value==k:
                sign = check_sign+' '
            btn[sign+self.text(v)] = f"{k}:multi_{name}"
        self.send(msg,btn,close=close,col=1)
        return -1
    e.callback=act
    # self.node('name',[e])
    #############################
    e1=Node()
    e1.pattern = self.check_inline_keyboards(f'multi_{name}')
    def act(self:Conversation,*a):
        tid = int(self.splited_query_data()[1])
        value = int(str(self.db.do(table,condition=cond)[0][target]))


        if tid== value:return -1
        
        try:ctid=target_type(tid)
        except:ctid=tid
        self.db.do(table,{target:ctid},condition=cond)
        self.user_info = self.get_user_info_from_db([])
        value = int(str(self.db.do(table,condition=cond)[0][target]))
        if value==None:value=0
        btn = {}
        for k,v in items.items():
            sign=''
            if value==k:
                sign = '✅ '
            btn[sign+self.text(v)] =  f"{k}:multi_{name}"
        self.edit_message_reply_markup(btn,self.get_msg_id(),col=1,close=close)
        return -1
    e1.callback=act
    p.entry=[e,e1]
    return p
    # self.node(f'sub{name}',[e])


from starco.tlg.bot.classes import Conversation
from starco.tlg.bot.util.enum import *
from starco.tlg.bot.util.filteres import *
import time
# from .functions import translator
from starco.utils import translator

##############################3
def translate(self:Conversation):
    
    e= Node()
    e.filters = match_btn('translate',self)
    def act(self:Conversation,*a):
        btn ={k:f'{v}:src_lang' for k,v in self.languages.items()}
        self.send('select_from',btn)
        return self.stat_key('translate',0)
    e.callback =act

    s=Node()
    s.pattern = self.check_inline_keyboards('src_lang')
    def act(self:Conversation,*a):
        print(2)
        self.userdata('src_lang',int(self.splited_query_data()[1]))
        btn ={k:f'{v}:trgt_lang' for k,v in self.languages.items()}
        self.edit_message_text('select_target',self.get_msg_id(),btn)
        return self.stat_key('translate',1)
    s.callback=act

    s1=Node()
    s1.pattern = self.check_inline_keyboards('trgt_lang')
    def act(self:Conversation,*a):
        self.delete_message(self.get_msg_id())
        self.wait()
        trgt_lang = int(self.splited_query_data()[1])
        src_lang = self.userdata('src_lang')
        texts = self.db.do('texts',condition=f"role={self.role[0]} AND language={src_lang}")
        ex_texts = [i['key'] for i in self.db.do('texts',condition=f"role={self.role[0]} AND language={trgt_lang}")]
        texts =[i for i in texts if i['key'] not in ex_texts]
        out=[]
        langs_pfx = {v:k for k,v in self.languages.items()}
        target = langs_pfx[trgt_lang]
        for i in texts:
            value = i['value']
            #do translat
            value = translator(value,target)
            if value==None:continue
            i['value']=value
            i['language'] = trgt_lang
            for role in self.role:
                i['role'] = role
                i['id']=self.time()
                out+=[i.copy()]
                time.sleep(0.01)
        self.db.do('texts',out)
        self.send('done')
        return -1
    s1.callback=act
    self.node('translate',[e],[[s],[s1]])


from threading import Thread
from functools import partial


def SendToAllBasic(self:Conversation):
    p=Pack()
    p.name = 'send_to_all_basic'

    e = Node()
    e.filters = match_btn('send_to_all', self)
    e.msg = 'enter_your_message'
    e.btn = [[self.back_menu_key]]

    s = Node()
    s.filters = Filters.all
    def act(self: Conversation, *args):
        msg_id, chat_id = self.get_msg_id(), self.get_chat_id()
        Thread(target=partial(self.send_to_users, msg_id, chat_id,None,[self.id])).start()
        self.send_message('message_is_sending', self.menu_keys)
        return -1
    s.callback = act

    p.entry=[e]
    p.states=[[s]]
    return p

#************** not completed********************
def SupportAdmin(self:Conversation):
    p = Pack()
    p.name = 'SupportAdmin'
    p.db_config={'support': {'id':0,'u_id': 0, 'msg_id': 0, 'response_id': 0, 'responser_id': 0, 'extra': '', 'status': 0, 'time': 0}}
    e=Node()
    e.filters = match_btn('support',self)

    def show_message_from_user(self:Conversation, item):
        try:

            lnk = None
            u_username = self.user('username')
            if u_username != None:
                lnk = f'https://t.me/{u_username}'
            try:
                name = self.user('name')[:13]
            except:
                name = 'info'
            btn = {
                'send_reply': f"{item['id']}",
                'user_info': f"{item['u_id']}",
                'mark_read': f"{item['id']}",
                'show_response': f"{item['id']}",
                'perv_sup_pm': f"{item['id']}:{item['u_id']}",
                'next_sup_pm': f"{item['id']}:{item['u_id']}",
            }
            tbtn = btn.copy()
            tbtn[name] = f"shinfo:{item['id']}"
            if lnk:
                tbtn['pv'] = lnk
            
            try:
                self.copy_message(from_chat_id=item['u_id'],message_id=item['msg_id'],chat_id=self.id,btns=tbtn)
            except:
                try:
                    btn['info'] = f"shinfo:{item['time']}"
                    if lnk:
                        btn['pv'] = lnk
                        self.copy_message(from_chat_id=item['u_id'],message_id=item['msg_id'],chat_id=self.id,btns=tbtn)
                except:
                    self.send('unreadable_message'+f"\n/user_{item['u_id']}", btn)
                    self.db.do('support', {'status': 1},condition=f"id={item['id']}")
        except Exception as e:
            self.send_message('unreadable_message'+f"\n/user_{item['u_id']}\nerror")
            self.db.do('support', {'status': 1},condition=f"id={item['id']}")
            self.debug.debug(e)


    def unreaded_messages(self:Conversation, *args):
        unreaded_messages = self.db.do('support', condition=f"status=0")
        self.send('wait', self.menu_btn)
        try:
            if not unreaded_messages:
                self.send('no_item', self.menu_keys)
            else:
                for item in unreaded_messages:
                    self.show_message_from_user(item)
        except Exception as e:
            self.debug.debug(e)
        return -1
    e.callback = unreaded_messages
    p.entry=[e]
    return p


    
    



import re
def get_number(txt):
    txt = str(txt)
    numbers =re.findall(r'\d+', txt)
    if numbers:
        sn = ''.join(numbers)
        sn = sn.lstrip('0')
        return int(sn)

def session_number(number:int):
    return f"+{number}.session"



import phonenumbers
from phonenumbers.phonenumberutil import region_code_for_country_code
from phonenumbers.phonenumberutil import region_code_for_number
import pycountry

def get_country_name(phone_number):
    try:
        phone_number = str(phone_number)
        if phone_number[0]!='+':phone_number=f"+{phone_number}"
        pn = phonenumbers.parse(phone_number)
        country = pycountry.countries.get(alpha_2=region_code_for_number(pn)).name
        country = country.replace(' ','_').replace(',','')
        return country
    except:return 'other'





import requests,json
def money_show(price,dec=0):
    price = round(float(str(price)),dec)
    return f"{price:,}"

import math
def round_decimals_up(number:float, decimals:int=2):
    """
    Returns a value rounded up to a specific number of decimal places.
    """
    if not isinstance(decimals, int):
        raise TypeError("decimal places must be an integer")
    elif decimals < 0:
        raise ValueError("decimal places has to be 0 or more")
    elif decimals == 0:
        return math.ceil(number)

    factor = 10 ** decimals
    return math.ceil(number * factor) / factor

def mask(string,n=-3):
    string = str(string)
    return string[:n].ljust(len(string), '*')

def fake_name():
    try:
        o = requests.get('https://api.namefake.com/').text
        return json.loads(o)['name']
    except:return 'Jewell Daniel'


def test_port(ip, port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(3)
    result = sock.connect_ex((ip, port))
    res = False
    if result == 0:
        res = True
    sock.close()
    return res

import socket
def get_ip_from_domain(domain:str):
    try:
        domain = domain.replace('*','asdfasf')
        return socket.gethostbyname(domain)
    except:pass

from deep_translator import GoogleTranslator
def translator(txt,dest,src='auto'):
    if dest=='ch':dest = 'zh-CN'
    try:return GoogleTranslator(source=src, target=dest).translate(txt)
    except Exception as e:print(e)




import sys,os
import traceback
from datetime import datetime
from..utils.directories import path_maker

class Debug:
    def __init__(self,debug_mode=True,relative_path='',log_name='log') -> None:
        self.debug_mode = debug_mode
        self.relative_path= relative_path
        self.log_name =log_name
    def debug(self,error):
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        pm = datetime.now().strftime("%H:%M") + f" , {fname}:{exc_tb.tb_lineno} => "
        try:
            if not isinstance(error,str):
                err= traceback.format_exc()
                errl = err.split('\n')
                err_msg = '\n\t'.join(errl[1:])
                pm+='\n'+err_msg
            else:
                error = str(error)
                pm+='\t'+error
            with open(path_maker([],self.relative_path)+'/'+self.log_name,'a+') as f:
                f.write(pm+'\n\n')
                if self.debug_mode:
                    print(pm)

        except:pass



import importlib


class Gateway:
    def __init__(self,gate_name:str,base_callback_url:str,gate_key:str,test_mode:bool=False,**kargs) -> None:
        '''
        'callback_method':str defualt set
        '''
        if gate_name not in Gateway.gates_list():
            raise Exception('wrong gate name')
        mod = importlib.import_module(f'gateways.gates.{gate_name}')
        self.gate = getattr(mod, 'gateway')(base_callback_url=base_callback_url,gate_key=gate_key,test_mode=test_mode,**kargs)
    
    @staticmethod
    def gates_list():
        return ['aqayepardakht','vandar','zibalpay','idpay','nextpay']
    def pay_link(self,amount:float,order_id:str=None,callback:str=None,**kargs):

        '''
            *idpay,nextpay=> need order_id*
        
            amount:toman

            callback or order_id must be filled

            mobile , email
        '''
        return self.gate.pay_link(amount=amount,callback=callback,order_id=order_id,**kargs)
    def verify(self,**kargs):
        '''
            aqayepardakht:amount,transid
            vandar       : token
            zibal        :track_id,order_id
            idpay        :id,order_id
            nextpay      :amount,trans_id

            return status,kargs
        '''
        return self.gate.verify(**kargs)

if __name__ == "__main__":
    g = Gateway('aqayepardakht','',True)
    pl = g.pay_link(1000,'https://test.ir')
    print(pl)



import glob
from random import choice, randint,randrange
from datetime import datetime
import asyncio
import os
from random import choice, random
import shutil
from time import sleep, time
from telethon.sync import TelegramClient
from telethon.sync import events, functions, errors
import telethon
from telethon.tl.functions.contacts import GetContactsRequest, ImportContactsRequest, DeleteContactsRequest, ResolveUsernameRequest,UnblockRequest,BlockRequest
from telethon.tl.functions.account import UpdateStatusRequest,SetPrivacyRequest
# from telethon.tl.functions.stats import 
from telethon.tl.types import InputPhoneContact, InputGeoPoint, InputPeerChannel, InputChannel, InputChatPhoto, InputPhoto, InputChatUploadedPhoto, InputFile, InputPeerUser,MessageEntityTextUrl

from telethon.tl.types import MessageEntityCode
# ======== token
from telethon.tl.functions.channels import GetParticipantsRequest
from telethon.tl.functions.help import GetUserInfoRequest
from telethon.tl.types import ReactionEmoji
from telethon.tl.types import InputChannel, ChannelAdminLogEventsFilter, ChannelParticipantsSearch
from telethon.tl.functions.account import UpdateProfileRequest

from telethon.tl.functions.channels import JoinChannelRequest, LeaveChannelRequest, CreateChannelRequest, CheckUsernameRequest, UpdateUsernameRequest, DeleteChannelRequest, EditPhotoRequest
from telethon.tl.functions.photos import UploadProfilePhotoRequest
from telethon.tl.functions.messages import GetMessagesViewsRequest,GetMessagesReactionsRequest,SendReactionRequest,SendVoteRequest
from telethon.tl.functions.users import GetUsersRequest,GetFullUserRequest
from telethon.tl.types import InputUserSelf
from telethon.tl.functions.messages import ImportChatInviteRequest
from telethon import functions, types
# from telethon.tl.functions.channels import 
# CheckUsernameRequest()
import os,json
from time import time
from starco.debug import Debug
from starco.utils import path_maker
from .utils import *
from starco.proxy import PROXY
import nest_asyncio
nest_asyncio.apply()
from starco.utils import translator



class TlgApp:
    def __init__(self, number, key='17203958', hash='82cefc4001e057c9d1488ab90e23d54f', loop=None, debug_mode=True,country_diring=False,reset_json=False,auto_connect=True,disable_proxy=False,just_data=False, **kwargs) -> None:
        '''
        relative_path='.'
        path_list =['accounts',number]
        js_path_list =['accounts',number]
        proxy={'proxy_type':'http,socks5','ip','port','username','password'}
        '''
        self.timeout=kwargs.get('timeout',10)
        self.proxy = kwargs.get('proxy')
        relative_path = kwargs.get('relative_path', '.')
        self.debug = Debug(debug_mode, relative_path='.')
        self.number_int = get_number(number)
        if not self.number_int :raise Exception('cant detect number from session_name')
        self.number = f"+{self.number_int}"
        self.auto_connect =   auto_connect   
        
        path_list:list = kwargs.get('path_list', ['accounts',str(self.number_int)])
        js_path_list= kwargs.get('js_path_list', ['accounts',str(self.number_int)])
        self.disable_proxy = disable_proxy
        if country_diring:
            path_list.insert(-1,get_country_name(self.number))
            js_path_list.insert(-1,get_country_name(self.number))

        self.base_path = path_maker(path_list, relative_path)
        self.json_path = path_maker(js_path_list, relative_path)+f"/{self.number}.json"
        if just_data:return
        if reset_json:
            try:os.remove(self.json_path)
            except:pass        
        self.key = key
        self.hash = hash

        self.path = f'{self.base_path}/{session_number(self.number_int)}'
        self.status = False
        if type(loop) == type(None):
            asyncio.get_event_loop().close()
            if asyncio.get_event_loop().is_closed():
                self.loop = asyncio.new_event_loop()
                asyncio.set_event_loop(self.loop)
                for i in glob.glob(f"{self.base_path}/*"):
                    if i in [self.json_path , self.path]:continue
                    try:os.remove(i)
                    except:pass

        self.client: TelegramClient = None
        self.init_client()
        self.js('last_check_time',int(time()))

        #################

        self.d = {}
        self.events = []
        self.last_schedual = 0  # time()
        self.sleep_min = 20
        self.checked_ids = []

    async def _get_status(self):
        try:
            res = await self.client.get_me()
            try:photo = res.photo.photo_id
            except:photo =None

            self.js('photo', photo)
            self.js('first_name', res.first_name)
            self.js('last_name', res.last_name)
            # self.js('bio', res.bio)
            self.js('username', res.username)
        except:
            return False
        if res == None:
            return False
        return True
   
    def connect(self):
        async def act(self: TlgApp):
            try:
                # if await self.client.is_user_authorized():
                await self.client.connect()
                return True
            except:pass
            return False

        self.loop.run_until_complete(act(self))

    def init_client(self):
        api_id = self.js('app_id')
        app_hash = self.js('app_hash')
        proxy = self.js('proxy')
        
        if self.proxy==None and proxy and proxy!='null':
            self.proxy=proxy

        if self.js('app_id'):
            self.key =api_id
            self.hash=app_hash

        system_versions = ['5.15.1','5.11.3','4.15.5','5.10.4','3.11.8','4.1.1']
        system_version =  choice(system_versions) if not self.js('system_version') else self.js('system_version')
        device_models =[f"Telegram Desktop {i}" for i in ['4.12.2','4.11.3','4.11.5','4.10.3','4.11.8','4.12.1']]
        device_model =  choice(device_models) if not self.js('device_model') else self.js('device_model')
        app_version =  '1.6.7' if not self.js('app_version') else self.js('app_version')
        extra = {
            'app_version': app_version,
            'device_model': device_model,
            'system_version': system_version,
            
        }
        if self.proxy and not self.disable_proxy:
            self.js('proxy',self.proxy)
            extra['proxy'] = PROXY(**self.proxy).make_proxy()

        self.client = TelegramClient(
            session=self.path, 
            api_id=self.key,
            api_hash=self.hash,
            timeout=self.timeout,
              **extra)
        self.status = True
        def setjs(self,k,v):
            if not self.js(k):
                self.js(k,v)
        if self.js('session_file')==None:
            setjs(self,'session_file',self.number)
            setjs(self,'phone',self.number)
            setjs(self,'register_time',int(time()))
            setjs(self,'app_id',self.key)
            setjs(self,'app_hash',self.hash)
            setjs(self,'sdk','sdk 23')
            setjs(self,'app_version',app_version)
            setjs(self,'device',device_model)
            setjs(self,'last_check_time',int(time()))
            setjs(self,'avatar',"null")
            setjs(self,'first_name',"null")
            setjs(self,'last_name',"null")
            setjs(self,'username',"null")
            setjs(self,'sex',"null")
            setjs(self,'lang_pack',"en")
            setjs(self,'system_lang_pack',"en")
            setjs(self,'ipv6',False)
            setjs(self,'password_2fa', '')
        setjs(self,'proxy', self.proxy)
        
        
        if self.auto_connect:            
            self.connect()

    def js(self,key,value=None,do_none=False):
        if self.json_path:
            # print(key,value)
            js_path = f"{self.json_path}"
            
            try:
                with open(js_path,'r') as f:
                    data = f.read()
                data =json.loads(data)
            except:data={}
            if type(value)==type(None) and not do_none:
                return data.get(key)
            data[key]=value
            with open(js_path,'w') as f:
                data = f.write(json.dumps(data))

    def get_status(self):
        return self.loop.run_until_complete(self._get_status())

    def disconnect(self):
        async def act(self):
            try:
                await self.client.disconnect()
                self.status = False
                return
            except:
                try:
                    await self.client.disconnect()
                    self.status = False
                    return
                except:
                    pass
        return self.loop.run_until_complete(act(self))

    def do_login(self, first_step=False, **args):
        '''
            steps: 0=> auto
                   1=>send code 
                   2=> enter code :args= code
                   3=> enter 2fa   :args= p2fa
        '''
        
        step = self.d.get('step')
        if first_step:
            step = 0
        if step == 0:
            self.d['code'] = ''
            self.d['hash'] = ''
            self.d['p2fa'] = ''
            step = 1
        hash = args.get('hash', '')
        code = str(args.get('code', ''))
        p2fa = str(args.get('p2fa', ''))
        if step == 1:
            if self.get_status():
                return 'session_is_active'
            out = self.loop.run_until_complete(self._send_code_request())
            if out[0]:
                self.d['step'] = 2
                self.d['hash'] = out[1]
                return ''
            return out[1]
        elif step == 2:
            if hash == '':
                hash = self.d.get('hash', '')
            out = self.loop.run_until_complete(self.sign_in(code, hash))
            if out[0]:
                self.d['step'] = 0
            if out[1] == 'need_pass2':
                self.d['step'] = 3
                self.d['code'] = code
                self.d['hash'] = hash
            return out[1]
        elif step == 3:
            if code == '':
                code = self.d.get('code', '')
            if hash == '':
                hash = self.d.get('hash', '')
            out = self.loop.run_until_complete(
                self.sign_in(code, hash, p2fa))
            if out[0]:
                self.d['step'] = 0
                self.js('password_2fa',p2fa)
            return out[1]
        self.d['step'] = 0
        return False, 'wrong parameter in (do_login)'

    def login_loop(self):
        res = self.do_login(first_step=True)
        if res == 'session_is_active':
            # print(res)
            return
        inp = input('code:')
        while True:
            res = self.do_login(code=inp)
            if res == 'need_pass2':
                inp = input('pass2:')
                break
            elif res != '':
                # print(res)
                inp = input('code:')
            else:
                print('success login')
                return

        while True:
            res = self.do_login(p2fa=inp)
            if res != '':
                # print(res)
                inp = input('pass2:')
            else:
                print('success login')
                return
    
    def send_code_request(self):
        return self.loop.run_until_complete(self._send_code_request())

    async def _send_code_request(self):
        '''
            if have error session been removed
        '''
        try:
            if not await self.client.is_user_authorized():
                auth = await self.client.send_code_request(self.number)
                auth_hahs = auth.phone_code_hash
                return True, auth_hahs
            else:
                return False, '⚠️خطا در ارسال کد !'
        except errors.PhoneNumberBannedError:
            return False, f'⚠️خطا در ارسال کد به شماره {self.number} شماره شما از تلگرام مسدود شده است !'
        except errors.PhoneNumberInvalidError:
            return False, f'⚠️ شماره {self.number} اشتباه است.'
        except errors.FloodWaitError as e3:
            return False, f'⏳ شماره {self.number} از سمت تلگرام محدود شده است و تا {e3.seconds} ثانیه دیگر قابل ثبت نیست.'
        except errors.PhoneNumberOccupiedError:
            return False, '⚠️خطا در ارسال کد !'
        except errors.PhoneNumberUnoccupiedError:

            return False, '⚠️خطا در ارسال کد !'
        except ConnectionError:
            return False, '❌ارور در ارسال کد لطفا دوباره امتحان کنید.'
        except Exception as e:
            self.debug.debug(e)
            return False, '⚠️خطا در ارسال کد !'

    async def sign_in(self, code, hash, p2fa=None):
        try:
            if not await self.client.is_user_authorized():
                await self.client.sign_in(phone=f'{self.number}', code=(code), phone_code_hash=hash)
                return True, ''
            else:
                return False, 'unknow_error'
        except errors.SessionPasswordNeededError as e:
            if p2fa:
                try:
                    await self.client.sign_in(password=str(p2fa))
                    return True, ''
                except Exception as e:
                    return False, "wrong_pass2"
            else:
                return False, "need_pass2"

        except errors.PhoneCodeExpiredError:

            return False, 'expied_code'
        except errors.PhoneCodeInvalidError:

            return False, 'wrong_code'
        except Exception as e:
            self.debug.debug(e)
            return False, 'unknow_error'

    async def sign_up(self, code, hash):
        try:
            fname = 'fname'
            lname = 'lname'

            await self.client.sign_up(code=code, first_name=fname, last_name=lname, phone_code_hash=hash)

            return True, ''
        except errors.SessionPasswordNeededError:

            return False, "need_pass2"
        except errors.PhoneCodeExpiredError:

            return False, 'expied_code'
        except errors.PhoneCodeInvalidError:

            return False, 'wrong_code'
        except Exception as e:
            self.debug.debug(e)
            return False, 'unknow_error'

    def get_telegram_code(self):
        async def act(self):
            try:
                async for dialog in self.client.iter_dialogs():
                    tmp = dialog.to_dict()
                    if tmp.get('_') == 'Dialog' and tmp.get('name') == 'Telegram':
                        code = translator(dialog.message.message,'en')
                        # if 'Login code' in code:
                        find = re.findall('(\d{5}).',code)
                        if find:
                            return find[0],dialog.message.date
                    # break
            except Exception as e:
                self.debug.debug(e)
        return self.loop.run_until_complete(act(self))

    def active_sessions(self):
        async def act(self: TlgApp):
            out=[]
            async with self.client as client:
                result = await client(functions.account.GetAuthorizationsRequest())
                for i in result.authorizations:
                    i = i.to_dict()
                    out+=[i]
            return out

        return self.loop.run_until_complete(act(self))

    def destroy_session(self,session_hash:int):
        async def act():
            try:
                await self.client(functions.account.ResetAuthorizationRequest(hash=int(session_hash)))
                return True
            except Exception as e:self.debug.debug(e)
            return False

        return self.loop.run_until_complete(act())

    def destroy_others_session(self):
        async def act(self: TlgApp):
            async with self.client as client:
                result = await client(functions.account.GetAuthorizationsRequest())
                for i in result.authorizations:
                    i = i.to_dict()
                    if not i['current']:
                        try:await client(functions.account.ResetAuthorizationRequest(hash=i['hash']))
                        except:pass

        return self.loop.run_until_complete(act(self))

    def remove_session(self):
        self.disconnect()
        try:shutil.rmtree(self.base_path)
        except:pass

    def just_this_sessions_is_active(self):
        async def act(self: TlgApp):
            res = False
            async with self.client as client:
                result = await client(functions.account.GetAuthorizationsRequest())
                current_exist = False
                for i in result.authorizations:
                    i = i.to_dict()
                    if i['current']:
                        current_exist = True
                res = len(result.authorizations) == 1 and current_exist
            return res
        return self.loop.run_until_complete(act(self))

    def do_change_or_set_2fa(self, new_p2fa: str,old_p2f=None):
        async def act():
            if not old_p2f:
                old_p2fa = self.js('password_2fa')
            else:old_p2fa=old_p2f
            if old_p2fa == '':
                try:
                    await self.client.connect()
                    await self.client.edit_2fa(new_password=new_p2fa)
                    self.js('password_2fa',new_p2fa)
                    return True
                except Exception as e:
                    self.debug.debug(e)
                    return False
            else:
                try:
                    if str(new_p2fa) != str(old_p2fa):
                        await self.client.connect()
                        await self.client.edit_2fa(str(old_p2fa), new_password=new_p2fa)
                        self.js('password_2fa',new_p2fa)
                        return True
                except Exception as e:
                    self.debug.debug(e)
                    return False
            return True

        return self.loop.run_until_complete(act())

    def change_info(self,  first_name=None, last_name=None, bio=None):
        async def act():
            try:
                await self.client(UpdateProfileRequest(first_name=first_name, last_name=last_name, about=bio))
                return True
            except Exception as e:
                self.debug.debug(e)
            return False

        return self.loop.run_until_complete(act())
    
    def change_profile_photo(self, photo_path:str):
        async def act():
            try:
                pic = await self.client.upload_file(photo_path)
                await self.client(UploadProfilePhotoRequest(file=pic))
                return True
            except Exception as e:
                self.debug.debug(e)

            return False

        return self.loop.run_until_complete(act())

    def send_message(self,entity, message,delete_dialog=False,json_formatting_entities=None,**kargs):
        '''
        file:str=file_path
        '''
        async def act():
            try:
                file= kargs.get('file')
                if file:
                    kargs.pop('file')
                    file=await self.client.upload_file(file)
                if json_formatting_entities:
                    out = []
                    for i in json.loads(json_formatting_entities):
                        if i['type']=='text_link':
                            out+=[MessageEntityTextUrl(i['offset'],i['length'],i['url'])]
                        elif i['type']=='code':
                            out+=[MessageEntityCode(i['offset'],i['length'])]
                        
                    kargs['formatting_entities'] = out

                res = await self.client.send_message(entity, message,file=file,**kargs,)
                if delete_dialog:
                    await self.client.delete_dialog(entity)
                return res
            except Exception as e:self.debug.debug(e)
        return self.loop.run_until_complete(act())
        
    def download(self,entity,size,out_file_name):
        async def act(self:TlgApp,entity,size,out_file_name):
            async for message in self.client.iter_messages(entity,limit=1, reverse=False):
                try:                
                    if type(message.media)!=type(None):
                        gsize=message.media.document.size
                        if size==gsize:
                            await self.client.download_media(message,out_file_name)
                            return True
                except:pass
            return False

        return self.loop.run_until_complete(act(self,entity,size,out_file_name))

    def upload(self,entity,path,caption,force_document=False):
        async def act(self:TlgApp,entity,path,caption,force_document):
            try:
                res = await self.client.send_file(entity,path,caption=caption,force_document=force_document)
                return res
            except:pass
        return self.loop.run_until_complete(act(self,entity,path,caption,force_document))
    
    def seen_post(self, entity, ids:list[int]=[],limit=5):

        async def _seen_post():
            '''
            if ids==[] use limit
            '''
            try:
                if not ids:
                    list_of_id = []
                    async for msg in self.client.iter_messages(entity, limit=limit):
                        list_of_id += [int(msg.id)]
                else:list_of_id = ids
                await self.client(GetMessagesViewsRequest(peer=entity, id=list_of_id, increment=True))

                return True
            except Exception as e:
                self.debug.debug(e)
            return False
            
        return self.loop.run_until_complete(_seen_post())
        
    def react_post(self, entity,reac:list, ids:list[int]=[],limit=5):
        if not isinstance(reac,list):reac=[reac]
        async def _react_post():
            '''
            if ids==[] use limit
            '''
            try:
                if not ids:
                    list_of_id = []
                    async for msg in self.client.iter_messages(entity, limit=limit):
                        list_of_id += [int(msg.id)]
                else:list_of_id = ids
                reaction=[ReactionEmoji(
                    emoticon=choice(reac)
                )]
                for msg_id in ids:
                    await self.client(SendReactionRequest(peer=entity,msg_id=msg_id,reaction=reaction))

                return True
            except Exception as e:
                self.debug.debug(e)
            return False
            
        return self.loop.run_until_complete(_react_post())
    
    def vote(self, entity, msg_id:int,option:str):
        async def act():
            try:
                await self.client(SendVoteRequest(peer=entity,msg_id=msg_id,options=[option.encode()]))
                return True
            except Exception as e:
                self.debug.debug(e)
            return False
            
        return self.loop.run_until_complete(act())
    
    def join_channel(self, channel):
        async def _join_channel():
            try:
                res = await self.client(JoinChannelRequest(channel))
                return True
            except Exception as e:
                self.debug.debug(e)
                return False
        return self.loop.run_until_complete(_join_channel())

    def leave_channel(self, channel):
        async def _leave_channel():
            try:
                await self.client(LeaveChannelRequest(channel))
                return True
            except Exception as e:
                self.debug.debug(e)
        return self.loop.run_until_complete(_leave_channel())
    
    def set_channel_photo(self, channel, pic):
        async def _set_channel_photo():
            try:
                upload_file_result = await self.client.upload_file(file=pic)
                input_chat_uploaded_photo = InputChatUploadedPhoto(
                    upload_file_result)
                await self.client(EditPhotoRequest(channel, input_chat_uploaded_photo))
                return True
            except Exception as e:
                self.debug.debug(e)
            return False
        return self.loop.run_until_complete(_set_channel_photo())
        
    def create_public_channel(self, title, username,  firt_msg=None):

        async def _create_public_channel():
            try:
                createdPrivateChannel = await self.client(CreateChannelRequest(title=title,
                                                                            about='',
                                                                            geo_point=InputGeoPoint(
                                                                                lat=1,
                                                                                long=2,
                                                                                accuracy_radius=42
                                                                            ),
                                                                            address='address'
                                                                            ))
                # if you want to make it public use the rest
                newChannelID = createdPrivateChannel.__dict__[
                    "chats"][0].__dict__["id"]
                newChannelAccessHash = createdPrivateChannel.__dict__[
                    "chats"][0].__dict__["access_hash"]
                desiredPublicUsername = username
                CH = InputPeerChannel(channel_id=newChannelID,
                                    access_hash=newChannelAccessHash)
                checkUsernameResult = await self.client(CheckUsernameRequest(InputPeerChannel(channel_id=newChannelID, access_hash=newChannelAccessHash), desiredPublicUsername))
                if (checkUsernameResult == True):
                    try:
                        publicChannel = await self.client(UpdateUsernameRequest(CH, desiredPublicUsername))
                    except:
                        pass
                    
                    if firt_msg:
                        await self.client.send_message(CH,firt_msg)
                    return True
                else:
                    await self.client(DeleteChannelRequest(CH))
            except Exception as e:
                self.debug.debug(e)
            return False

        return self.loop.run_until_complete(_create_public_channel())

    def get_all_dialogs(self):
        '''return groups entity'''
        async def act():
            ids = []
            try:
                async for dialog in self.client.iter_dialogs():
                    tmp = dialog.to_dict().get('entity', {})
                    ids += [tmp.to_dict()]
            except Exception as e:
                self.debug.debug(e)
            return ids
        return self.loop.run_until_complete(act())

    def get_channel_id(self, inputs):

        async def _get_channel_id():
            res = 0
            try:
                res = await self.client.get_entity(inputs)
                res = res.id
            except Exception as e:
                self.debug.debug(e)

            try:
                res = int(res)
            except:
                pass

            return res
        return self.loop.run_until_complete(_get_channel_id())

    def export_contacts(self,convert_to_dict=True):

        async def _export_contacts():
            try:
                contacts = await self.client(GetContactsRequest(0))
                if len(contacts.users) > 0:
                    res = []
                    for i in contacts.users:
                        if convert_to_dict:
                            i = i.to_dict()
                        res += [i]
                    return res
                return []
            except Exception as e:
                return []
        return self.loop.run_until_complete(_export_contacts())

    def import_contacts(self, list_number:list):

        async def _import_contacts():
            try:
                contacts = []
                for i in list_number:
                    number = '+'+(str(i).replace('+', ''))
                    contacts += [InputPhoneContact(client_id=randrange(-2**63, 2**63),
                                                phone=number, first_name=number, last_name='')]
                res = await self.client(ImportContactsRequest(contacts=contacts))

                return res

            except Exception as e:
                self.debug.debug(e)
            return []
        return self.loop.run_until_complete(_import_contacts())

    def delete_contacts(self, id:list):
        async def _delete_contacts():
            try:
                await self.client(DeleteContactsRequest(id))
                return True
            except Exception as e:
                self.debug.debug(e)
        return self.loop.run_until_complete(_delete_contacts())

    def get_all_groups(self):
        '''return groups entity'''
        async def _get_all_groups():
            ids = []
            try:
                async for dialog in self.client.iter_dialogs():
                    tmp = dialog.to_dict().get('entity', {})
                    if tmp:
                        entity = tmp
                        tmp = tmp.to_dict()
                        if tmp['_'] == 'Channel' and (tmp['gigagroup'] or tmp['megagroup']) and not tmp['restricted']:
                            ids += [entity]
            except Exception as e:
                self.debug.debug(e)
            return ids
        return self.loop.run_until_complete(_get_all_groups())

    def number_maker(self, count=1):
        pidh = ['911', '912', '933', '936', '937', '938', '939', '902', '990']
        res = []
        for _ in range(count):
            number = f"+98{choice(pidh)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}{randint(0,9)}"
            res += [number]
        return res

    def get_user_status_by_username(self, username):
        async def _get_user_status_by_username():
            try:
                res = await self.client.get_entity(username)
                status = res.status.to_dict()['_']
                return status
            except:
                pass
            return ''
        return self.loop.run_until_complete(_get_user_status_by_username())

    def on_new_message(self,func,**kargs):
        '''
        func(self:TlgApp,event,**kargs)
        '''
        @self.client.on(event=events.NewMessage)
        async def action(event):
            try:
                func(self,event,**kargs)
            except Exception as e:
                print(e)

        self.client.run_until_disconnected()

    def get_user_info(self, per_user,to_dict=False):
        '''
        send 1 username
        '''
        async def _get_user_info():
            userls = per_user
            if not isinstance(userls,list):userls=[userls]
            try:
                res = await self.client(GetUsersRequest(userls))
                if to_dict:
                    out=[]
                    for i in res:
                        out+=[i.to_dict()]
                    out=out[-1]
                    return out
                res=res[-1]
                return res
            except Exception as e:
                print(e)
            return {}
        return self.loop.run_until_complete(_get_user_info())

    def delete_old_dialoges(self,after_sec=864000):

        async def _delete_old_dialoges():
            out=0
            try:
                async for dialog in self.client.iter_dialogs():
                    tmp = dialog.to_dict()
                    if tmp.get('_') == 'Dialog':
                        if tmp['entity'].to_dict().get('_')!='User':continue
                        send_time = tmp['message'].to_dict()['date'].timestamp()
                        past = int((int(time()) - send_time))
                        if past > after_sec:
                            user = tmp['entity']
                            await self.client.delete_dialog(user)
                            out+=1


            except Exception as e:
                self.debug.debug(e)
            return out

        return self.loop.run_until_complete(_delete_old_dialoges())

    def copy_posts(self, from_id:int, to_id:int,offset_id:int, limit:int):
        '''return list of copied id'''
        async def _cp_posts():
            entity = {}
            out = []
            async for dialog in self.client.iter_dialogs():
                tmp = dialog.to_dict().get("entity", {})
                if tmp:
                    if tmp.to_dict().get('id') == from_id:
                        entity = tmp
                        break
            if entity:
                async for message in self.client.iter_messages(entity,offset_id=offset_id,limit=limit, reverse=True):
                    try:
                        id = message.to_dict().get('id')
                        await self.client.send_message(to_id, message)
                        out += [id]
                    except:
                        pass
            return out
        return self.loop.run_until_complete(_cp_posts())
        
    def get_members_info(self,username=None,gid=None,hash=None):
        '''
        username for public group
        id,access hash for private

        '''
        async def _get_group_members_info():
            out = []
            if username!=None:
                o= await self.client(ResolveUsernameRequest(username))
                id = o.chats[0].id
                access_hash = o.chats[0].access_hash
            else:
                id=gid
                access_hash=hash
            id = int(str(id))
            access_hash = int(str(access_hash))
            chat = InputChannel(id,access_hash)
            async for i in self.client.iter_participants( chat, limit=10000, aggressive=True):
                    try:
                        out += [i.to_dict()]
                    except:pass
            return out
        return self.loop.run_until_complete(_get_group_members_info())

    def join_private_group(self, group_hash_id):

        '''return id,access_hash'''
        async def _join_group():
            try:
                res = await self.client(ImportChatInviteRequest(group_hash_id))
                chats = res.to_dict().get('chats')
                if chats:
                    chat = chats[0]
                    id = chat['id']
                    access_hash=  chat['access_hash']
                return id,access_hash
            except Exception as e:
                print(e)
            return None,None
        return self.loop.run_until_complete(_join_group())
    
    def ative_last_seen(self):
        '''privacy'''
        async def act():
            try:
                await self.client(SetPrivacyRequest(types.InputPrivacyKeyStatusTimestamp(),[types.InputPrivacyValueDisallowAll()]))
                return True
            except Exception as e:
                print(e)
            return False
        return self.loop.run_until_complete(act())
    
    def remove_proxy(self):
        self.js('proxy',None,True)
        return True
    
    def disable_show_phone_number(self):
        '''privacy'''
        async def act():
            try:
                await self.client(
                    SetPrivacyRequest(
                        types.InputPrivacyKeyPhoneNumber(),
                        [types.InputPrivacyValueDisallowAll()]
                        ))
                return True
            except Exception as e:
                print(e)
            return False
        return self.loop.run_until_complete(act())
    
    def get_last_message(self,entity):
        async def act():
            try:
                out = await self.client.get_messages(entity)
                return out[0].message
            except Exception as e:
                self.debug.debug(e)
        return self.loop.run_until_complete(act())
    
    def get_me(self):
        async def act():
            try:
                out = await self.client.get_me()
                return out
            except Exception as e:
                self.debug.debug(e)
        return self.loop.run_until_complete(act())

    def get_account_status(self):
        '''
        return 'limit' , limit string time
        return 'ok' , ''
        return 'ban,''
        '''
        ban_pm='Unfortunately, some phone numbers may trigger a harsh response from our anti-spam systems'
        self.send_message('SpamBot','/start')
        sleep(2)
        last_message = self.get_last_message('SpamBot')
        print(last_message)
        if 'your account is now limited' in last_message:
            pattern = "your account is now limited until (.*?UTC)"
            stime = re.findall(pattern,last_message)[0]
            return 'limit' , stime
        elif 'Unfortunately, some actions can trigger a harsh' in last_message:
            return 'suspand' ,''
        elif 'Good news, no limits are currently applied to your account' in last_message:
            return 'ok',''
        elif ban_pm in last_message:
            return 'ban',''
                

    def do_online(self):
        async def act(self:TlgApp):
            res = await self.client(UpdateStatusRequest(False))
            return res
        return self.loop.run_until_complete(act(self))
    
    def st_online_status(self,online=True):
        async def act(self:TlgApp):
            res = await self.client(UpdateStatusRequest(not online))
            return res
        return self.loop.run_until_complete(act(self))
    def log_out(self):
        async def act(self:TlgApp):
            res = await self.client.log_out()
            return res
        return self.loop.run_until_complete(act(self))

    def unblock(self,uid:str):
        async def act(self:TlgApp):
            res = await self.client(UnblockRequest(uid))
            return res
        return self.loop.run_until_complete(act(self))

    def block(self,uid:str):
        async def act(self:TlgApp):
            res = await self.client(BlockRequest(uid))
            return res
        return self.loop.run_until_complete(act(self))


import requests,os
from starco.debug import Debug
from time import sleep
class WpApi:
    def __init__(self,username:str,password:str,site_url:str) -> None:
        ''''
            site_url = https://your_site.com
        '''
        self.username = username
        self.password = password
        self.site_url = site_url.rstrip('/')+'/'
        self.token=None
        self.status = False
        self.init_token()  
        self.debug = Debug(relative_path='.')
    
    def check_token(self):
        res = self.req('posts',get=True)
        if res.status_code==200:return True
        return False
    
    def fetch_token(self):
        try:
            data = {
                'username':self.username,
                'password':self.password
            }
            res= self.req('token',data,'api/v1',False).json()
            self.token = res['jwt_token']
        except:pass
        
    def req(self,end_point:str,data:dict=None,end_point_pfx='wp/v2',set_header = True,get=False):
        try:
            # posts
            headers=None
            if set_header:
                headers = {
                    "Authorization": f"Bearer {self.token}",
                    "Content-Type": "application/json"
                }
            url = self.site_url+f'wp-json/{end_point_pfx}/{end_point}'
            if get:
                return requests.get(url,json=data,headers=headers,timeout=10)
            return requests.post(url,json=data,headers=headers,timeout=10)
        except Exception as e:
            self.debug.debug(e)
    def init_token(self):
        path='.token'
        if os.path.exists(path):
            with open(path,'r') as f:
                self.token = f.read()
            if self.check_token():
                self.status=True
                return        
        self.fetch_token()
        if self.check_token():
            with open('.token','w') as f:
                f.write(self.token)
            self.status=True
            return 
        
    def posts(self,title=None,content=None,status='publish',**args):
        '''
            *if any parameters dont be fill return all posts
            featured_media:int
            tags:list:str
            categories:list:str
        '''
        if title:
            data = {
                "title": title,
                "content": content,
                "status": status
            }
            tags=args.get('tags',[])
            categories=args.get('categories',[])
            featured_media=args.get('featured_media')
            
            if tags:
                o_tags=[]
                for i in tags:
                    o_tags+=self.meta(i,type='tags')
                if o_tags:
                    data['tags'] = o_tags
            if categories:
                o_categories=[]
                for i in categories:
                    o_categories+=self.meta(i,type='categories')
                    
                if o_categories:
                    data['categories'] = o_categories
            if featured_media:
                data['featured_media'] = featured_media
            res= self.req('posts',data)
        else:
            res= self.req('posts',get=True)
        return res
    
    def meta(self,name:str=None,type='tags'):
        '''
            type =['tags','categories']
            *if any parameters dont be fill return all type
            else return id
        '''
        name = name.strip()
        try:
            if name:
                o= self.req(type,data={'search':name},get=True)
                o = [i['id'] for i in o.json() if i['name']==name]
                if o:return o
                data = {
                    "name": name,
                }
                res= self.req(type,data)
                return [res.json()['id']]
            else:
                res= self.req(type,get=True)
            return res
        except Exception as e:
            self.debug.debug(e)
            return []

if __name__ == "__main__":
    cls = WpApi(
        'manmodiram',
        'XGwr&25htW$G(Y1uhC',
        'http://starco.fun',
    )
    
            
    o =cls.posts('title۲۲','test۲۳۴۵۶۴',categories=['کانفیگ رایگان V2ray'])
    # o=[i['id'] for i in o.json() if i['name']=='test']
    print(o.json())


# response = requests.post(url, json=data, headers=headers)





from enum import Enum

# class syntax
class Role(Enum):
    SUPERADMIN = 0
    ADMIN = 1
    USER = 2
    ACCOUNTER= 3
    ROLE_A=4
    ROLE_B=5
    ROLE_C=6


class ConversationNode():
    entries=[]
    states={}
    fallbacks=[]
    arges = {}

class Node:
    command=None
    command_filters=None
    filters=None
    pattern=None
    callback=None
    btn=None
    msg=None
    chat_id=0
    send_args={}
    args={}
    state_idx=None

class Pack:
    name:str
    entry:list[Node]
    states:list[Node]=None
    fallbacks:list[Node]=None
    db_config:dict[dict]={}
    kargs={}
    
class HeadPack:
    name:str
    func=None
    db_config:dict[dict]={}
    pack:Pack=None
    
   

DEPOSIT = 1
WITHDRAW=2
ADMIN_CHARGE=3
ORDER_PRODUCT=4
EXPENS1=5
EXPENS2=6
EXPENS3=7
EXPENS4=8

WAITING=0
CONFIRMED=1
REJECTED=2
REQUESTED=3

BTN_PFX='_btn'




from cryptography.fernet import Fernet
import pickle,os,json
from time import time

class Pkl:
    def __init__(self, path='setting', encrypt=False, real_time=True):
        self.data = {}
        self.path = path
        self.real_time = real_time
        self.encryption = False
        self.BASE_ENCODE_KEY = '95AuiJk_wGQT9xcEZ6Xlymgh67LZyxHt34LShTkRO9w='
        
        if encrypt:
            self.encryption = True
            self.fernet = Fernet(self.BASE_ENCODE_KEY.encode())

    def reset(self):
        try:
            os.remove(self.path)
            self.data = {}
            return True
        except:
            pass
        return False

    def pkl(self, key=None, value=None, empty_return='', expir_sec: int = 0):
        if type(key) != type(None):
            pfx= '_set_timefpkl'
            if pfx in key:
                return
            set_time_label = key+pfx
        if not self.data:
            try:
                with open(self.path, "rb") as f:
                    save_data = pickle.load(f)
                    if self.encryption:
                        save_data = json.loads(
                            self.fernet.decrypt(save_data).decode())
            except Exception as e:
                save_data = {}
        else:
            save_data = self.data
        
            
        if type(value) != type(None):
            save_data[key] = value
            save_data[set_time_label] = int(time())
            if self.real_time:
                self.data = save_data
            with open(self.path, "wb") as fh:
                if self.encryption:
                    save_data = self.fernet.encrypt(
                        json.dumps(save_data).encode())
                pickle.dump(save_data, fh)
        elif type(key) != type(None):
            expir_time = 0
            if expir_sec > 0:
                set_time = save_data.get(set_time_label, 0)
                if set_time != 0:
                    expir_time = set_time + expir_sec
            if expir_time > 0:
                if int(time()) > expir_time:
                    return empty_return
            return save_data.get(key, empty_return)
        else:
            return save_data


from nowpayments import NOWPayments
from nowpayments.sandbox import NOWPaymentsSandbox

class NowPayment:
    def __init__(self,api,sandbox=False) -> None:
        self.api = api
        self.payment= NOWPayments(self.api)
        if sandbox:
            self.payment=NOWPaymentsSandbox(self.api)
    
    def create_payment(self,amount:float,price_currency:str='USD',crypto = 'usdttrc20',**args):
        out = self.payment.create_payment(amount,price_currency,crypto,**args)
        return out
    def check(self,payment_id):
        return self.payment.get_payment_status(payment_id)


    
from time import time,sleep

class Scheduler:
    def __init__(self,super_self) -> None:
        self.super_self = super_self
        self.schecdaul_info={}

    def allow(self,name):
        now = int(time())
        item = self.schecdaul_info.get(name,{})
        last_run = item.get('last_run',0)
        first_run = item.get('first_run',0)
        run_evry_sec = item.get('run_evry_sec',0)
        if last_run==0:
            self.schecdaul_info[name]['last_run']=now
            if first_run:
                return True
        else:
            if now%run_evry_sec==0 or now - last_run > run_evry_sec:
                self.schecdaul_info[name]['last_run']=now
                return True
                
        return False

    def add(self,func,run_evry_sec:int,first_run=False):
        '''
            function with input self
        '''
        name = str(func.__name__)
        self.schecdaul_info[name] = {'func':func,'run_evry_sec':run_evry_sec,'first_run':first_run,'last_run':0}
    
    def run(self):
        while True:
            for name , dict_val in self.schecdaul_info.items():
                try:
                    if self.allow(name):
                        dict_val['func'](self.super_self)
                except Exception as e:self.super_self.debug.debug(e)
            sleep(1)




import os

def path_maker(path_list=[],relative_path='.',start_path=None):
    '''
    start_path=None=>os.getcwd()
    '''
    start_path = os.getcwd() if start_path==None else start_path
    p = os.path.abspath(os.path.join(start_path,relative_path))
    for i in path_list:
        p += '/'+str(i)
        if not os.path.exists(p):
            os.mkdir(p)
    return p
import zipfile
def zipfolder(zip_addr:str, target_dir): 
    if not zip_addr.endswith('.zip'):
        zip_addr=zip_addr + '.zip'
    zipobj = zipfile.ZipFile(zip_addr, 'w', zipfile.ZIP_DEFLATED)
    rootlen = len(target_dir) + 1
    for base, dirs, files in os.walk(target_dir):
        for file in files:
            fn = os.path.join(base, file)
            zipobj.write(fn, fn[rootlen:])
    return zip_addr

def unziper(path_to_zip_file,directory_to_extract_to):
    with zipfile.ZipFile(path_to_zip_file, 'r') as zip_ref:
        zip_ref.extractall(directory_to_extract_to)


from starco.utils import path_maker,zipfolder,unziper
import os,shutil
from starco.tlg.app.utils import get_number
from starco.tlg.bot.util.enum import *
from starco.tlg.bot.util.filteres import *
from starco.tlg.bot.classes import Conversation
from telegram.ext import Filters
import time
from starco.tlg.app import TlgApp
def setuper(value:str,subset:str,unit:str='',type:str='',hide:int=0,toggle:int=0):
    return {'value':value,'subset':subset,'unit':unit,'type':type,'hide':hide,'toggle':toggle}

def SpamBot_req(number):
    tlg = TlgApp(number)
    time.sleep(1)
    status,_=tlg.get_account_status()
    if status!='ban':
        tlg.disconnect()
        return
    time.sleep(2)
    tlg.get_telegram_code
    tlg.send_message('SpamBot','Submit a complaint')
    time.sleep(2)
    tlg.send_message('SpamBot','No, I’ll never do any of this!')
    time.sleep(7)
    pm='Dear Telegram Support!\nMy Telegram Account has been spammed suddenly and I cannot sent message to any contacts whom I don’t have their number and I want you to help me and fix the issue and remove my number from blacklist.'
    tlg.send_message('SpamBot',pm)
    tlg.disconnect()

def backup_database(self,chat_id=''):
    path = self.db.path
    database_dir = path_maker(['database'])
    shutil.copy2(path,database_dir)
    zip_path = path_maker([],'.')+'/database.zip'
    zipfolder(zip_path,database_dir)
    with open(zip_path,'rb') as f:
        if chat_id=='':chat_id = self.super_admin
        self.bot.send_document(chat_id, f)
    try:os.remove(zip_path)
    except:pass
    try:shutil.rmtree(database_dir)
    except:pass

def copy_ready_session(zip_path,saved_before):
    rootdir=path_maker(['zipdir'],'.')
    unziper(zip_path,rootdir)
    sessions={}
    for root, subdirs, files in os.walk(rootdir):
        for filename in files:
            number = get_number(filename)
            if number in saved_before:
                print('duplicated')
                continue
            if number and number>0:
                file_path = os.path.join(root, filename)
                sessions[number]=sessions.get(number,{})
                if filename.endswith('.session'):
                    sessions[number]['session']=file_path
                elif filename.endswith('.json'):
                    sessions[number]['json']=file_path
    ow_path=lambda number,x:os.path.join(path_maker(['accounts',number],'.'),f"+{(x.split('/')[-1]).lstrip('+')}")
    print(sessions)
    for number , values in sessions.items():
        if 'session' not in values or 'json' not in values:continue
        print(values['session'],ow_path(number,values['session']))
        print(values['json'],ow_path(number,values['json']))
        shutil.move(values['session'],ow_path(number,values['session']))
        shutil.move(values['json'],ow_path(number,values['json']))

    shutil.rmtree(rootdir)
    os.remove(zip_path)
    return list(sessions.keys())

def get_product(self,id:int,status=CONFIRMED):
    try:
        return self.db.do('products',condition=f"id={id} AND status={status}")[0]
    except:return {}

def get_orders_by_pid(self,pid:int,status=WAITING):
    try:
        return self.db.do('orders',condition=f"pid={pid} AND status={status}")
    except:return []

def get_order(self,id:int,status=WAITING):
    try:
        return self.db.do('orders',condition=f"id={id} AND status={status}")[0]
    except:return {}

def setup_force_join(self):
    e = Node()
    e.filters = match_btn('force_join', self)

    def act(self: Conversation, *args):
        force_joins = self.db.do('force_join')
        if force_joins:
            self.send('lists', [['add_force_join'], [self.back_menu_key]])

            for i in force_joins:
                msg = f"{self.text('title',slash=True)}: {i['title']}\n"
                msg += f"{self.text('channel_id',slash=True)}: {i['channel_id']}"
                msg += f"{self.text('link',slash=True)}: {i['link']}"
                self.send(msg, {'delete_fj': i['id']}, t=False)
        else:
            self.send('no_item', [['add_force_join'], [self.back_menu_key]])
        return -1
    e.callback = act
    self.node('force_join', [e])
    ########################################
    e = Node()
    e.pattern = self.check_inline_keyboards('delete_fj')

    def act(self: Conversation, *args):
        id = int(self.splited_query_data()[1])
        self.db.do('force_join', delete=True)
        self.edit_message_text('deleted', self.get_msg_id())
        return -1
    e.callback = act
    self.node('delete_fj', [e])
    ########################################
    e = Node()
    e.filters = match_btn('add_force_join', self)
    e.msg = 'enter_title'
    e.btn = [[self.back_menu_key]]

    s = Node()
    s.filters = Filters.text
    s.msg = 'enter_channel_id'
    s.btn = [[self.back_menu_key]]

    s1 = Node()
    s1.filters = Filters.text
    s1.msg = 'enter_channel_link'
    s1.btn = [[self.back_menu_key]]

    s2 = Node()
    s2.filters = Filters.text
    def act(self: Conversation, *args):
        title = self.userdata(self.stat_key('add_force_join', 1))['text']
        channel_id = self.userdata(self.stat_key('add_force_join', 2))['text']
        link = self.get_text()
        self.db.do('force_join', {'id': self.time(), 'title': title,'channel_id':channel_id, 'link': link,'status':CONFIRMED})
        self.send('done', self.menu_keys)
        return -1

    s2.callback = act
    self.node('add_force_join', [e], [[s],[s1], [s2]])




from setuptools import setup,find_packages
import os,glob
requires = [
    'cffi==1.15.1',
    'cryptography==41.0.3',
    'pycparser==2.21',
    'beautifulsoup4==4.12.2',
    'lxml==4.9.3',
    'requests==2.31.0',
    'easy-db==0.9.15',
    'hcloud==1.32.0',
    'cloudflare==2.11.6',
    'python-socks==2.4.3',
    'python-telegram-bot==13.14',
    'Telethon==1.33.1',
    'phonenumbers',
    'pycountry',
    'nest-asyncio==1.5.8',
    'zibal',
    'nowpayments',
    'jdatetime',
    'deep-translator',
    'async-timeout',
    'PySocks',
    'selenium==4.17.2',
    'pyperclip==1.8.2',
    
]

setup(
    name = 'Starco',long_description='starco project',
    version='3.1.8',
    author='Mojtaba Tahmasbi',
    packages=find_packages(),
    install_requires=requires,
)



import requests,json
from .utils import number_maker,get_random_email,Gate

class gateway(Gate):
    def __init__(self ,**kargs) -> None:
        super().__init__(**kargs)
        if self.test_mode:
            self.gate_key='6a7f99eb-7c20-4412-a972-6dfb7cd253a4'

    def pay_link(self,**args):
        amount,callback,order_id = args.get('amount') , args.get('callback'), args.get('order_id')
        if not callback:
            callback = self.callback_maker(order_id)

        amount*=10
        mobile,email = args.get('mobile',str(number_maker())) , args.get('email',str(get_random_email()))
        url = 'https://api.idpay.ir/v1.1/payment'
        data = {
            'order_id': order_id,
            'amount': amount,
            'callback': callback,
            'phone':mobile,
            'mail':email
            
        }
        headers = {
            'Content-Type': 'application/json',
            'X-API-KEY': self.gate_key,
            'X-SANDBOX': '1' if self.test_mode else '0'
        }
        request = requests.post(url, data = json.dumps(data), headers = headers,timeout=20)
        print(request.text)
        if request.status_code == 201:
            return request.json()['link']


    def inquiry(self, id, order_id):

        url = 'https://api.idpay.ir/v1.1/payment/inquiry'

        data = {
            'id': id,
            'order_id': order_id,
        }

        headers = {
            'Content-Type': 'application/json',
            'X-API-KEY': self.gate_key,
            'X-SANDBOX': '1' if self.test_mode else '0'
        }

        request = requests.post(url, data=json.dumps(data), headers=headers,timeout=20)

        if request.status_code == 200:
            return True

        return False

    
    def verify(self,**args):
        id,order_id = args.get('id') , args.get('order_id')
        
        url = 'https://api.idpay.ir/v1.1/payment/verify'
        data = {
            'id': id,
            'order_id': order_id,
        }

        headers = {
            'Content-Type': 'application/json',
            'X-API-KEY': self.gate_key,
            'X-SANDBOX': '1' if self.test_mode else '0'
        }

        request = requests.post(url, data=json.dumps(data), headers=headers,timeout=20)
        try:args['amount'] = float(args['amount'])/10
        except:pass
        if request.status_code == 200:
            return self.inquiry(id,order_id) ,args
        return False ,args








# class IDPayAPI:

#     def __init__(self, api_key, domain, sandbox = False):
#         self.api_key = api_key
#         self.domain = domain
#         self.sandbox = sandbox


#     def payment(self, order_id, amount, callback_page, payer = {}):

     
#     def verify(self, id, order_id):




#     
#     def get_status(self, status):

        states = {
            1: 'Transaction created',
            2: 'Transaction failed',
            3: 'An error has occurred',
            4: 'Transaction blocked',
            5: 'Transaction rejected to payer',
            6: 'Transaction rejected',
            7: 'Transaction canceled',
            8: 'Redirected to IPG',
            10: 'Verify pending',
            100: 'Transaction verified',
            101: 'Verified again',
            200: 'Transaction settled',
        }

        if states[int(status)]:
            return states[int(status)]

        return False


from telegram.ext.filters import MessageFilter,UpdateFilter,BaseFilter
from.enum import Role
from telegram import Update
from telegram.ext import Filters
from .enum import CONFIRMED


class match_text(MessageFilter):
    def __init__(self, target) -> None:
        super(match_text, self).__init__()
        self.target = target

    def filter(self, message):
        return self.target == message.text

class IsReplied(MessageFilter):
    def __init__(self,*args) -> None:
        super(IsReplied, self).__init__()
    def filter(self, message):
        out = message.reply_to_message!=None and message.chat.id==message.reply_to_message.chat.id
        return out

class special_user(MessageFilter):
    def __init__(self, target) -> None:
        super(special_user, self).__init__()
        self.target = target

    def filter(self, message):
        return self.target == message.from_user.id
    
class match_btn(MessageFilter):
    def __init__(self, target,super_self) -> None:
        super(match_btn, self).__init__()
        self.target = target
        self.super_self = super_self

    def filter(self, message):
        res = self.super_self.replace_btn_label(self.target, main_word= message.text) == message.text
        return res

class RoleFilter(UpdateFilter):
    """Represents a filter that has been inverted.

    Args:
        f: The filter to invert.

    """

    __slots__ = ('f',)

    def __init__(self, f,*args):
        self.f = f

    def filter(self, update) -> bool:
        if not update.message:return False
        try:
            super_self = self.f
            u_id = update.message.from_user.id
            user = super_self.db.do('users',condition=f"id={u_id}")
            if user:
                res = user[0]['role'] in super_self.role
                return res
            if u_id == super_self.super_admin:
                return True
            return Role.USER.value in super_self.role
        except Exception as e:
            super_self.debug.debug(e)
        return True
 
class FileFormat(UpdateFilter):
    """Represents a filter that has been inverted.

    Args:
        f: The filter to invert.

    """

    __slots__ = ('format')

    def __init__(self, format:str,*args):
        self.format:str = format

    def filter(self, update:Update) -> bool:
        try:
            format = self.format.lower()
            file_path =update.effective_message.effective_attachment
            if file_path:
                file_path=file_path.get_file().file_path
                file_format = file_path.split('.')[-1].lower()
                if format==file_format:
                    return True
        except Exception as e:print(e)
        return False
    
class ForceJoinFilter(UpdateFilter):
    __slots__ = ('super_self')
    def __init__(self, super_self:str,*args):
        self.super_self = super_self

    def filter(self, update:Update) -> bool:
        try:
            super_self = self.super_self
            uid =update.message.from_user.id
            channels = super_self.db.do('force_join',condition=f"status={CONFIRMED}")
            if len(channels) > 0:
                for ch in channels:
                    channel_id = ch['channel_id']
                    try:
                        if not super_self.bot.getChatMember(channel_id, uid)['status'] in ['member', 'administrator', 'creator']:
                            return False
                    except Exception as e:
                        print(2,e)
        except Exception as e:print(1,e)
        return True

class SelectLanguageFilter(UpdateFilter):
    __slots__ = ('super_self')
    def __init__(self, super_self:str,*args):
        self.super_self = super_self

    def filter(self, update:Update) -> bool:
        try:
            ulanguage = self.super_self.user('language')
            uid = update.message.from_user.id
            if ulanguage==None:
                self.super_self.user_info=self.super_self.get_user_info_from_db([],uid)
                ulanguage = self.super_self.user('language')
            languages = self.super_self.languages
            if  ulanguage in [self.super_self.defulat_lang_code,None] and languages:
                return True
        except Exception as e:print(1,e)
        return False

class SharePhoneFilter(UpdateFilter):
    __slots__ = ('super_self')
    def __init__(self, super_self:str,*args):
        self.super_self = super_self

    def filter(self, update:Update) -> bool:
        try:
            phone = self.super_self.user('phone')
            if  phone in [0,None] and not Filters.contact(update):
                return True
        except Exception as e:print(1,e)
        return False


import requests,json
from .utils import number_maker,get_random_email,Gate

class gateway(Gate):
    def __init__(self ,**kargs) -> None:
        super().__init__(**kargs)
        if self.test_mode:
            self.gate_key='b11ee9c3-d23d-414e-8b6e-f2370baac97b'

 
    def pay_link(self,**args):
        amount,callback,order_id = args.get('amount') , args.get('callback'), args.get('order_id')
        if not callback:
            callback = self.callback_maker(order_id)
        mobile,email = args.get('mobile',str(number_maker())) , args.get('email',str(get_random_email()))
        payload=f'api_key={self.gate_key}&amount={amount}&customer_phone={mobile}&payer_desc={email}&order_id={order_id}&callback_uri={callback}&auto_verify=yes'
        headers = {
        'User-Agent': 'PostmanRuntime/7.26.8',
        'Content-Type': 'application/x-www-form-urlencoded'
        }
        url = "https://nextpay.org/nx/gateway/token"
        
        response = requests.request("POST", url, headers=headers, data=payload)

        print(response.text)
        json_data = json.loads(response.text)
        if response.status_code == 200 and str(json_data['code']) == '-1':
            return f"https://nextpay.org/nx/gateway/payment/{json_data['trans_id']}"
        return None            
    
    def verify(self,**args):
        amount,trans_id = args.get('amount') , args.get('trans_id')
        url = "https://nextpay.org/nx/gateway/verify"

        payload=f'api_key={self.gate_key}&trans_id={trans_id}&amount={amount}'
        headers = {
        'User-Agent': 'PostmanRuntime/7.26.8',
        'Content-Type': 'application/x-www-form-urlencoded'
        }

        response = requests.request("POST", url, headers=headers, data=payload)
        print(response.text)
        json_data = json.loads(response.text)
        if response.status_code == 200 and str(json_data['code']) == '0':
            return True ,args
        return False  ,args


from functools import wraps
# from selenium import webdriver
from lib2to3.pgen2 import driver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver import *
# from selenium.webdriver.firefox.service import Service as FService
# from selenium.webdriver.chrome.options import Options as CHOptions
# from selenium.webdriver.firefox.options import Options as FOptions
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from platform import system
import os
from time import sleep
from starco.utils import path_maker
from starco.pkl import Pkl
import json

class WScraper:
    def __init__(self, name,url, browser='firefox',background=False,use_binary_location =True,profile_dirname=None) -> None:
        self.url = url
        self.browser = browser
        self.background=background
        self.binary_location = use_binary_location
        self.profile_dirname =profile_dirname
        self.driver = self.init_driver()
        self.pkl = Pkl(path_maker()+f'/{name}').pkl
        self.coockies = None
        
    
    def dec_chk_login(func):
        @wraps(func)
        def magic(self, *args, **kw):
            try:
                return func(self, *args, **kw)
            except Exception as e:
                print(e)
        return magic

    def driver_path(self):
      
        system_name = system().lower()
        base = path_maker(['drivers',f'{system_name}_{self.browser}'])
        path_maker(['drivers',f'{system_name}_{self.browser}','profile_dirname'])
        if self.browser == 'chrome':
            driver_name = 'chromedriver'
            bin_name=f'{self.browser}'
        elif self.browser == 'firefox':
            driver_name = 'geckodriver'
            bin_name=f'{self.browser}-bin'
        if  system_name== 'linux':pass
        elif system_name == 'windows':
            driver_name+='.exe'
            bin_name+='.exe'
        else:
            raise Exception('Not work on this os')
        
        

        return base , driver_name,bin_name
    
    def install_addon(self, path, temporary=None):
        # Usage: driver.install_addon('/path/to/firebug.xpi')
        # ‘self’ refers to the “Webdriver” class
        # 'path' is absolute path to the addon that will be installed
        payload = {"path": path}
        if temporary:
            payload["temporary"] = temporary
        # The function returns an identifier of the installed addon.
        # This identifier can later be used to uninstall installed addon.
        return self.driver.execute("INSTALL_ADDON", payload)["value"]

    def init_driver(self):
        base_path,driver_name ,bin_name= self.driver_path()
        print(base_path)
        if self.browser == 'chrome':
            service = ChromeService
            option = ChromeOptions
            web_driver =Chrome
            # fp=webdriver.Chrome.
        elif self.browser == 'firefox':
            service = FirefoxService
            option = FirefoxOptions
            web_driver = Firefox
            
        
        cfg= {'service':service(base_path+'/'+driver_name)}
        options:FOptions = option()
        if self.binary_location:options.binary_location = base_path+'/'+bin_name
        if self.background:options.add_argument("--headless")
        if profile_dirname:=self.profile_dirname:
            if self.browser == 'firefox':
                fp = FirefoxProfile(f'{base_path}/{profile_dirname}')
                options.profile = fp
            if self.browser == 'chrome':
                
                options.add_argument(f"--user-data-dir={base_path}/user_data") #e.g. C:\Users\You\AppData\Local\Google\Chrome\User Data
                options.add_argument(f'--profile-directory={profile_dirname}') #e.g. Profile 3
            options.add_argument("--enable-javascript")

        # options.add_argument(f"--user-data-dir={ud}") #e.g. C:\Users\You\AppData\Local\Google\Chrome\User Data
        # options.add_argument(f'--profile-directory=profiles') 
        # .add_extension('')
        cfg['options']=options
        return web_driver(**cfg)

    def save_cookies(self):
        self.pkl('cookies', self.driver.get_cookies())
    
    def save_local_storage(self):
        self.pkl('local_storage', self.driver.execute_script("return window.localStorage;"))
    

    def load_local_storage(self):
        local_storage = self.pkl('local_storage', empty_return={})
        for k,v in local_storage.items():
            if v:
                self.driver.execute_script(f"window.localStorage.setItem('{k}',{json.dumps(v)});")
        self.load_url()

    def load_cookies(self):
        cookies = self.pkl('cookies', empty_return=[])
        if len(cookies) > 0:
            for cookie in cookies:
                try:
                    self.driver.add_cookie(cookie)
                except Exception as e:
                    print(cookie)
            self.coockies = cookies
            self.load_url()

    def check_page_loaded(self):
        WebDriverWait(self.driver, 15).until(EC.element_to_be_clickable((By.CSS_SELECTOR, 'html')))
        
    def check_element(self, by:By, element):
        try:
            self.driver.find_element(by, element)
            return True
        except:
            pass
        return False
   
    def load_url(self,url=''):
        if url =='':url =self.url
        self.driver.get(url)
        self.check_page_loaded()
       
    def fill_input(self,by:By,targe:str,value:str):
        elem = WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((by, targe)))
        return elem.send_keys(value)
    
    def click(self,by:By,targe:str):
        elem = WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((by, targe)))
        return elem.click()
    
    def get_text(self,by:By,targe:str):
        elem = WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((by, targe)))
        return elem.text
    def get_html(self,by:By,targe:str):
        elem = WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((by, targe)))
        return elem.get_attribute('innerHTML')
     
    def get_class(self,by:By,targe:str,clickable=True):
        return self.get_attr(by,targe,'class',clickable)
    
    def get_attr(self,by:By,targe:str,attr,clickable=True):
        if clickable:
            elem = WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((by, targe)))
        else:elem =self.driver.find_element(by, targe)
        return elem.get_attribute(attr)


